import { TestBed } from '@angular/core/testing';

import { ProtocolService } from './protocol.service';

import { _MODULE_CONFIG } from '../config/token';
import { WebSocketService } from '@ucap/ng-web-socket';
import { LogService } from '@ucap/ng-logger';
import { of } from 'rxjs';
import { SVC_TYPE_PING, SSVC_TYPE_PING_REQ } from '@ucap/protocol-ping';

const MockModuleConfig = {
  requestId: {
    min: 0,
    max: 0
  },
  timeout: {
    expiredTime: 0,
    checkInterval: 0
  }
};

describe('ProtocolService', () => {
  let service: ProtocolService;
  let logService: LogService;
  let webSocketServiceSpy: jasmine.SpyObj<WebSocketService>;

  beforeEach(() => {
    const spyWebSocketService = {
      ...jasmine.createSpyObj('WebSocketService', ['connect']),
      socketMessage$: of()
    };

    logService = new LogService({});

    TestBed.configureTestingModule({
      providers: [
        ProtocolService,
        { provide: _MODULE_CONFIG, useValue: MockModuleConfig },
        { provide: WebSocketService, useValue: spyWebSocketService },
        { provide: LogService, useValue: logService }
      ]
    });
    service = TestBed.inject(ProtocolService);
    webSocketServiceSpy = TestBed.inject(WebSocketService) as jasmine.SpyObj<
      WebSocketService
    >;
  });

  it('should be created', () => {
    expect(service).toBeTruthy();
  });

  it('service connect', async () => {
    await service.connect();
    expect(service).toBeTruthy();
  });

  it('service send', () => {
    try {
      service.send(SVC_TYPE_PING, SSVC_TYPE_PING_REQ, ...[]);
      logService.debug('s send');
    } catch (e) {
      console.log(e);
    }

    expect(service).toBeTruthy();
  });
});
