import { of } from 'rxjs';
import { map, catchError, exhaustMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { RegViewResponse } from '@ucap/protocol-option';
import { AuthResponse } from '@ucap/protocol-query';

import { QueryProtocolService } from '@ucap/ng-protocol-query';
import { OptionProtocolService } from '@ucap/ng-protocol-option';

import * as LoginActions from '../login/actions';

import {
  auth,
  authSuccess,
  authFailure,
  regView,
  regViewSuccess,
  regViewFailure
} from './actions';

@Injectable()
export class Effects {
  sessionCreatedForAuth$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(LoginActions.sessionCreated),
      map((action) =>
        auth({ req: { deviceType: action.loginSession.deviceType } })
      )
    );
  });

  auth$ = createEffect(() =>
    this.actions$.pipe(
      ofType(auth),
      map((action) => action.req),
      exhaustMap((req) => {
        return this.queryProtocolService.auth(req).pipe(
          map((res: AuthResponse) => {
            return authSuccess({
              userPermission: res.userPermission
            });
          }),
          catchError((error) => of(authFailure({ error })))
        );
      })
    )
  );

  sessionCreatedForRegView$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(LoginActions.sessionCreated),
      map((action) => regView({ req: {} }))
    );
  });

  regView$ = createEffect(() =>
    this.actions$.pipe(
      ofType(regView),
      map((action) => action.req),
      exhaustMap((req) => {
        return this.optionProtocolService.regView(req).pipe(
          map((res: RegViewResponse) => {
            return regViewSuccess({
              settings: res.settings
            });
          }),
          catchError((error) => of(regViewFailure({ error })))
        );
      })
    )
  );

  constructor(
    private actions$: Actions,
    private queryProtocolService: QueryProtocolService,
    private optionProtocolService: OptionProtocolService
  ) {}
}
