import { of } from 'rxjs';
import { catchError, map, switchMap, tap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';

import { UserActions } from '@ucap/ng-store-organization';

import { logout, logoutSuccess, loginSuccess } from './actions';

@Injectable()
export class Effects {
  logout$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(logout),
      switchMap((action) => {
        return this.authenticationProtocolService.logout({}).pipe(
          map((res) => logoutSuccess({ res })),
          catchError((error) => of(error))
        );
      })
    );
  });

  loginSuccessForOrganizationUserInit$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(loginSuccess),
        map((result) => result.loginInfo),
        tap((loginInfo) => {
          this.store.dispatch(
            UserActions.init({
              user: {
                info: loginInfo.userInfo,
                companyCode: loginInfo.companyCode,
                departmentCode: loginInfo.departmentCode,
                statusMessage1: loginInfo.statusMessage1,
                statusMessage2: loginInfo.statusMessage2,
                statusMessage3: loginInfo.statusMessage3,
                madn: loginInfo.madn,
                hardPhoneSadn: loginInfo.hardPhoneSadn,
                fmcSadn: loginInfo.fmcSadn,
                pbxIndex: loginInfo.pbxIndex,
                talkWithMeBotSeq: loginInfo.talkWithMeBotSeq
              }
            })
          );
        })
      ),
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private authenticationProtocolService: AuthenticationProtocolService
  ) {}
}
