import { Injectable, Inject } from '@angular/core';

import { of } from 'rxjs';
import { map, catchError, exhaustMap } from 'rxjs/operators';
import { Actions, createEffect, ofType } from '@ngrx/effects';

import {
  CallHistoryInfoResponse,
  CallHistoryDelResponse
} from '@ucap/api-contact';
import { ContactApiService } from '@ucap/ng-api-contact';
import { StatusCode } from '@ucap/api';

import {
  callHistory,
  callHistorySuccess,
  callHistoryFailure,
  del,
  delSuccess,
  delFailure
} from './actions';
import { ModuleConfig } from '../../config/module-config';
import { _MODULE_CONFIG } from '../../config/token';

@Injectable()
export class Effects {
  callHistory$ = createEffect(() =>
    this.actions$.pipe(
      ofType(callHistory),
      map((action) => action.req),
      exhaustMap((req) => {
        return this.contactApiService.callHistoryList(req).pipe(
          map((res: CallHistoryInfoResponse) => {
            if (res.statusCode === StatusCode.Success) {
              const remainHistory =
                !!res.historyList &&
                res.historyList.length >=
                  this.moduleConfig.historyRequestDefaultCount
                  ? true
                  : false;
              return callHistorySuccess({
                res,
                remainHistory
              });
            } else {
              return callHistoryFailure({ error: res.errorMessage });
            }
          }),
          catchError((error) => of(callHistoryFailure({ error })))
        );
      })
    )
  );

  del$ = createEffect(() =>
    this.actions$.pipe(
      ofType(del),
      map((action) => action.req),
      exhaustMap((req) => {
        const callLogSeq = !!req.targetReceiverCallLogSeq
          ? Number(req.targetReceiverCallLogSeq)
          : !!req.targetSenderCallLogSeq
          ? Number(req.targetSenderCallLogSeq)
          : undefined;

        if (!!callLogSeq) {
          return this.contactApiService.callHistoryDel(req).pipe(
            map((res: CallHistoryDelResponse) => {
              if (res.statusCode === StatusCode.Success) {
                return delSuccess({
                  callLogSeq
                });
              } else {
                return delFailure({ error: 'error statusCode' });
              }
            }),
            catchError((error) => of(delFailure({ error })))
          );
        } else {
          return of(delFailure({ error: 'not target calllogseq' }));
        }
      })
    )
  );

  constructor(
    private actions$: Actions,
    @Inject(_MODULE_CONFIG) private moduleConfig: ModuleConfig,
    private contactApiService: ContactApiService
  ) {}
}
