import { Selector, createSelector } from '@ngrx/store';

import { SearchDirectionByDateType, CallHistory } from '@ucap/domain-call';
import { EntityState, createEntityAdapter } from '@ngrx/entity';

export interface CallHistoryState extends EntityState<CallHistory> {}
export const adapterCallHistory = createEntityAdapter<CallHistory>({
  selectId: (callHistory) => callHistory.callLogSeq,
  sortComparer: (a, b) => {
    return b.callLogSeq - a.callLogSeq;
  }
});
const callHistoryInitialState: CallHistoryState = adapterCallHistory.getInitialState();
const {
  selectAll: selectAllForCallHistory,
  selectEntities: selectEntitiesForCallHistory,
  selectIds: selectIdsForCallHistory,
  selectTotal: selectTotalForCallHistory
} = adapterCallHistory.getSelectors();

export interface State {
  searchDate?: string | null;
  searchDirection?: SearchDirectionByDateType | null;
  remainHistory?: boolean | null;
  historyList: CallHistoryState;
}

export const initialState: State = {
  searchDate: null,
  searchDirection: null,
  remainHistory: null,
  historyList: callHistoryInitialState
};

export function selectors<S>(selector: Selector<any, State>) {
  const selectCallHistory = createSelector(
    selector,
    (state: State) => state.historyList
  );

  return {
    callHistory: createSelector(selectCallHistory, selectAllForCallHistory),
    searchDate: createSelector(selector, (state: State) => state.searchDate),
    searchDirection: createSelector(
      selector,
      (state: State) => state.searchDirection
    )
  };
}
