import { createReducer, on } from '@ngrx/store';

import { Info, FileInfo, EventType, EventJson } from '@ucap/domain-chat';

import * as RoomActions from '../room/actions';

import {
  initialState,
  adapterChatting,
  adapterEventList,
  Chatting,
  adapterFileInfoList
  // adapterFileInfoCheckList
} from './state';

import {
  eventsSuccess,
  eventsFailure,
  fileInfosSuccess,
  fileInfosFailure,
  addEvent,
  delEventList,
  updateEventList,
  clearActiveRoomId
} from './actions';

export const reducer = createReducer(
  initialState,

  on(eventsSuccess, (state, action) => {
    const roomId = action.roomEventInfo.roomId;

    const chatting = state.chattings.entities[roomId] || {};
    let trgtChatting: Chatting = {
      roomId,

      eventListProcessing: false,
      eventList: adapterEventList.getInitialState(),
      eventStatus: null,
      remainEvent: null,

      fileInfoListProcessing: false,
      fileInfoList: adapterFileInfoList.getInitialState(),
      // fileInfoCheckList: adapterFileInfoCheckList.getInitialState(),
      fileInfoCheckList: [],
      fileInfoSyncDate: '',
      ...chatting
    };

    // dupliaction event process
    const trgtEventInfoList: Info<EventJson>[] = [];
    if (!!chatting && !!chatting.eventList && !!chatting.eventList.entities) {
      const filteredList = action.eventInfoList.filter((item) => {
        let notExistOrDiff = true; // added target flag.

        // tslint:disable-next-line: forin
        for (const key in chatting.eventList.entities) {
          const event = chatting.eventList.entities[key];
          if (
            item.seq === event.seq &&
            item.type === event.type &&
            item.sentMessage === event.sentMessage
          ) {
            notExistOrDiff = false;
            break;
          }
        }

        return notExistOrDiff;
      });

      if (!!filteredList && filteredList.length > 0) {
        trgtEventInfoList.push(...filteredList);
      }
    } else {
      trgtEventInfoList.push(...action.eventInfoList);
    }

    trgtChatting = {
      ...trgtChatting,
      eventList: adapterEventList.upsertMany(trgtEventInfoList, {
        ...trgtChatting.eventList
      }),
      eventStatus: action.roomEventInfo,
      remainEvent:
        trgtChatting.remainEvent === false ? false : action.remainEvent, // 재조회를 위한 처리.
      eventListProcessing: false
    };

    return {
      ...state,
      chattings: adapterChatting.upsertOne(trgtChatting, {
        ...state.chattings
      })
    };
  }),

  on(eventsFailure, (state, action) => {
    const roomId = action.roomId;

    const chatting = state.chattings.entities[roomId];
    let trgtChatting: Chatting;
    if (!!chatting) {
      trgtChatting = {
        ...chatting,
        eventListProcessing: false
      };
    }

    return {
      ...state,
      chattings: adapterChatting.upsertOne(trgtChatting, {
        ...state.chattings
      })
    };
  }),

  on(fileInfosSuccess, (state, action) => {
    const roomId = action.res?.roomId;

    if (!roomId) {
      return state;
    }

    const chatting = state.chattings.entities[roomId] || {};
    let trgtChatting: Chatting = {
      roomId,

      eventListProcessing: false,
      eventList: adapterEventList.getInitialState(),
      eventStatus: null,
      remainEvent: null,

      fileInfoListProcessing: false,
      fileInfoList: adapterFileInfoList.getInitialState(),
      // fileInfoCheckList: adapterFileInfoCheckList.getInitialState(),
      fileInfoCheckList: [],
      fileInfoSyncDate: '',
      ...chatting
    };

    const fileInfoList = action.fileInfoList;
    const fileInfoCheckList = action.fileInfoCheckList;

    trgtChatting = {
      ...trgtChatting,
      fileInfoList: !!fileInfoList
        ? adapterFileInfoList.upsertMany(fileInfoList, {
            ...trgtChatting.fileInfoList
          })
        : trgtChatting.fileInfoList,
      // fileInfoCheckList: !!fileInfoCheckList
      //   ? adapterFileInfoCheckList.upsertMany(fileInfoCheckList, {
      //       ...trgtChatting.fileInfoCheckList
      //     })
      //   : trgtChatting.fileInfoCheckList,
      fileInfoCheckList,
      fileInfoListProcessing: false
    };

    return {
      ...state,
      chattings: adapterChatting.upsertOne(trgtChatting, {
        ...state.chattings
      })
    };
  }),

  on(fileInfosFailure, (state, action) => {
    const roomId = action.roomId;

    const chatting = state.chattings.entities[roomId];
    let trgtChatting: Chatting;
    if (!!chatting) {
      trgtChatting = {
        ...chatting,
        fileInfoListProcessing: false
      };
    }

    return {
      ...state,
      chattings: adapterChatting.upsertOne(trgtChatting, {
        ...state.chattings
      })
    };
  }),

  on(addEvent, (state, action) => {
    const roomId = action.roomId;
    const eventInfo = action.info;

    const chatting = state.chattings.entities[roomId];

    if (!!chatting) {
      return {
        ...state,
        chattings: adapterChatting.upsertOne(
          {
            ...chatting,
            eventList: adapterEventList.upsertOne(eventInfo, {
              ...chatting.eventList
            })
          },
          { ...state.chattings }
        )
      };
    } else {
      return state;
    }
  }),

  on(delEventList, (state, action) => {
    const roomId = action.roomId;
    const chatting = state.chattings.entities[roomId];

    // checked Valid array
    if (
      !chatting ||
      !chatting.eventList ||
      chatting.eventList.ids.length === 0
    ) {
      return state;
    }

    const fileInfoList = chatting.fileInfoList;
    const trgtDelFileInfoSeqs = [];
    if (!!fileInfoList) {
      fileInfoList.ids.forEach((id) => {
        const fileInfo: FileInfo = fileInfoList.entities[id];
        if (action.eventSeqs.indexOf(fileInfo.eventSeq) > -1) {
          trgtDelFileInfoSeqs.push(id);
        }
      });
    }

    return {
      ...state,
      chattings: adapterChatting.upsertOne(
        {
          ...chatting,
          eventList: adapterEventList.removeMany(action.eventSeqs, {
            ...chatting.eventList
          }),
          fileInfoList:
            trgtDelFileInfoSeqs.length > 0
              ? adapterFileInfoList.removeMany(trgtDelFileInfoSeqs, {
                  ...chatting.fileInfoList
                })
              : chatting.fileInfoList
        },
        { ...state.chattings }
      )
    };
  }),

  on(updateEventList, (state, action) => {
    const roomId = action.roomId;
    const eventSeq = action.eventSeq;
    const sentMessage = action.sentMessage;
    const chatting = state.chattings.entities[roomId];
    let fileInfoSeq;

    // checked Valid array
    if (
      !chatting ||
      !chatting.eventList ||
      chatting.eventList.ids.length === 0
    ) {
      return state;
    }

    const statusEventInfo: Info<EventJson> = {
      ...chatting.eventList[eventSeq],
      type: EventType.RecalledMessage,
      sentMessage
    };

    // 파일이 회수되었을 경우 fileInfoList 에서도 삭제 한다.
    const currentEvent = chatting.eventList.entities[eventSeq];
    if (!!currentEvent && currentEvent.type === EventType.File) {
      const fileInfoList = chatting.fileInfoList;
      if (!!chatting && !!fileInfoList) {
        fileInfoList.ids.forEach((id) => {
          const fileInfo: FileInfo = fileInfoList.entities[id];
          if (!!fileInfo && action.eventSeq === fileInfo.eventSeq) {
            fileInfoSeq = id;
          }
        });
      }
    }

    return {
      ...state,
      chattings: adapterChatting.upsertOne(
        {
          ...chatting,
          eventList: adapterEventList.updateOne(
            {
              id: eventSeq,
              changes: statusEventInfo
            },
            {
              ...chatting.eventList
            }
          ),
          fileInfoList: !!fileInfoSeq
            ? adapterFileInfoList.removeOne(fileInfoSeq, {
                ...chatting.fileInfoList
              })
            : chatting.fileInfoList
        },
        { ...state.chattings }
      )
    };
  }),

  on(clearActiveRoomId, (state, action) => {
    return {
      ...state,
      activeRoomId: null
    };
  }),

  /*******************************************************************
   * [Room Action watching.]
   *******************************************************************/
  on(RoomActions.selectedRoomSuccess, (state, action) => {
    return {
      ...state,
      activeRoomId: action.roomId
    };
  }),

  on(RoomActions.clearSelectedRoom, (state, action) => {
    if (action.roomId === state.activeRoomId) {
      return {
        ...state,
        activeRoomId: null
      };
    } else {
      return state;
    }
  }),

  on(RoomActions.close, (state, action) => {
    const roomIds = action.roomIds;

    return {
      ...state,
      chattings: adapterChatting.removeMany(roomIds, {
        ...state.chattings
      })
    };
  })
);
