import { Selector, createSelector } from '@ngrx/store';
import { EntityState, createEntityAdapter } from '@ngrx/entity';

import {
  Info,
  FileInfo,
  FileDownloadInfo,
  EventJson,
  RoomEventInfo
} from '@ucap/domain-chat';

export interface EventListState extends EntityState<Info<EventJson>> {}
export interface FileInfoListState extends EntityState<FileInfo> {}
// export interface FileInfoCheckListState extends EntityState<FileDownloadInfo> {}

export const adapterEventList = createEntityAdapter<Info<EventJson>>({
  selectId: (info) => info.seq,
  sortComparer: (a, b) => {
    return a.seq - b.seq;
  }
});
export const adapterFileInfoList = createEntityAdapter<FileInfo>({
  selectId: (info) => info.seq,
  sortComparer: (a, b) => {
    return b.seq - a.seq;
  }
});
// export const adapterFileInfoCheckList = createEntityAdapter<FileDownloadInfo>({
//   selectId: (info) => info.seq,
//   sortComparer: (a, b) => {
//     return b.seq - a.seq;
//   }
// });

const eventListInitialState: EventListState = adapterEventList.getInitialState(
  {}
);
const fileInfoListInitialState: FileInfoListState = adapterFileInfoList.getInitialState(
  {}
);
// const fileInfoCheckListInitialState: FileInfoCheckListState = adapterFileInfoCheckList.getInitialState(
//   {}
// );

const {
  selectAll: selectAllForEventList,
  selectEntities: selectEntitiesForEventList,
  selectIds: selectIdsForEventList,
  selectTotal: selectTotalForEventList
} = adapterEventList.getSelectors();

const {
  selectAll: selectAllForFileInfoList,
  selectEntities: selectEntitiesForFileInfoList,
  selectIds: selectIdsForFileInfoList,
  selectTotal: selectTotalForFileInfoList
} = adapterFileInfoList.getSelectors();

// const {
//   selectAll: selectAllForFileInfoCheckList,
//   selectEntities: selectEntitiesForFileInfoCheckList,
//   selectIds: selectIdsForFileInfoCheckList,
//   selectTotal: selectTotalForFileInfoCheckList
// } = adapterFileInfoCheckList.getSelectors();

export interface Chatting {
  roomId?: string;

  eventListProcessing?: boolean;
  eventList?: EventListState;
  eventStatus?: RoomEventInfo | null;
  remainEvent?: boolean | null;

  fileInfoListProcessing?: boolean;
  fileInfoList?: FileInfoListState;
  // fileInfoCheckList?: FileInfoCheckListState;
  fileInfoCheckList?: FileDownloadInfo[];
  fileInfoSyncDate?: string;
}

export interface ChattingState extends EntityState<Chatting> {}

export const adapterChatting = createEntityAdapter<Chatting>({
  selectId: (chatting) => chatting.roomId
});

const chattingInitialState: ChattingState = adapterChatting.getInitialState({});

const {
  selectAll: selectAllForChatting,
  selectEntities: selectEntitiesForChatting,
  selectIds: selectIdsForChatting,
  selectTotal: selectTotalForChatting
} = adapterChatting.getSelectors();

export interface State {
  chattings: ChattingState;
  activeRoomId: string | null;
}

export const initialState: State = {
  chattings: chattingInitialState,
  activeRoomId: null
};

export function selectors<S>(selector: Selector<any, State>) {
  const selectChattings = createSelector(
    selector,
    (state: State) => state.chattings
  );

  const selectChatting = createSelector(
    selectChattings,
    (state: ChattingState, roomId: string) => {
      return state.entities && state.entities[roomId];
    }
  );

  const selectChattingEventListProcessing = createSelector(
    selectChatting,
    (state) => state.eventListProcessing
  );
  const selectChattingEventList = createSelector(
    selectChattings,
    (state: ChattingState, roomId: string) => {
      const chatting = state.entities && state.entities[roomId];
      if (!!chatting) {
        return chatting?.eventList;
      } else {
        return adapterEventList.getInitialState();
      }
    }
  );
  const selectChattingEventStatus = createSelector(
    selectChatting,
    (state) => state.eventStatus
  );
  const selectChattingRemainEvent = createSelector(
    selectChatting,
    (state) => state.remainEvent
  );
  const selectChattingFileInfoListProcessing = createSelector(
    selectChatting,
    (state) => state.fileInfoListProcessing
  );
  const selectChattingFileInfoList = createSelector(
    selectChattings,
    (state: ChattingState, roomId: string) => {
      const chatting = state.entities && state.entities[roomId];
      if (!!chatting) {
        return chatting?.fileInfoList;
      } else {
        return adapterFileInfoList.getInitialState();
      }
    }
  );
  // const selectChattingFileInfoCheckList = createSelector(
  //   selectChattings,
  //   (state: ChattingState, roomId: string) => {
  //     const chatting = state.entities && state.entities[roomId];
  //     if (!!chatting) {
  //       return chatting?.fileInfoCheckList;
  //     } else {
  //       return adapterFileInfoCheckList.getInitialState();
  //     }
  //   }
  // );
  const selectChattingFileInfoCheckList = createSelector(
    selectChattings,
    (state: ChattingState, roomId: string) => {
      const chatting = state.entities && state.entities[roomId];
      if (!!chatting) {
        return chatting?.fileInfoCheckList;
      } else {
        return [];
      }
    }
  );
  const selectChattingFileInfoSyncDate = createSelector(
    selectChatting,
    (state) => state.fileInfoSyncDate
  );

  return {
    activeRoomId: createSelector(selector, (state) => state.activeRoomId),

    chattings: createSelector(selectChattings, selectAllForChatting),
    chatting: selectChatting,

    eventListProcessing: selectChattingEventListProcessing,
    eventList: createSelector(selectChattingEventList, selectAllForEventList),
    eventStatus: selectChattingEventStatus,
    remainEvent: selectChattingRemainEvent,

    fileInfoListProcessing: selectChattingFileInfoListProcessing,
    fileInfoList: createSelector(
      selectChattingFileInfoList,
      selectAllForFileInfoList
    ),
    // fileInfoCheckList: createSelector(
    //   selectChattingFileInfoCheckList,
    //   selectAllForFileInfoCheckList
    // ),
    fileInfoCheckList: selectChattingFileInfoCheckList,
    fileInfoSyncDate: selectChattingFileInfoSyncDate
  };
}
