import { createAction, props } from '@ngrx/store';

import { LocaleCode } from '@ucap/domain-common';
import { RoomInfo, UserInfo, UserInfoShort } from '@ucap/domain-chat';

import {
  InfoRequest as RoomRequest,
  InviteNotification,
  OpenRequest as CreateRequest,
  OpenResponse as CreateResponse,
  Open3Request as CreateTimerRequest,
  Open3Response as CreateTimerResponse,
  ExitRequest as DeleteRequest,
  ExitResponse as DeleteResponse,
  ExitAllRequest as DeleteMultiRequest,
  ExitAllResponse as DeleteMultiResponse,
  UpdateRequest,
  UpdateResponse,
  InviteRequest,
  InviteResponse,
  ExitForcingRequest,
  ExitForcingResponse,
  UpdateTimerSetRequest,
  UpdateTimerSetResponse,
  InfoResponse
} from '@ucap/protocol-room';

/**
 * retrieve selected Room.
 */
export const selectedRoom = createAction(
  '[ucap::chat::selectedRoom] selected room',
  props<{ roomId: string; localeCode: LocaleCode }>()
);
/**
 * retrieve selected Room Success.
 */
export const selectedRoomSuccess = createAction(
  '[ucap::chat::selectedRoom] selected room validation Success',
  props<{
    roomId: string;
    roomInfo2Res: {
      roomInfo: RoomInfo;
      roomUserInfo: {
        userInfoList: UserInfo[];
        userInfoShortList: UserInfoShort[];
      };
      res: InfoResponse;
    };
  }>()
);
/**
 * clear selected Room.
 */
export const clearSelectedRoom = createAction(
  '[ucap::chat::clearSelectedRoom] clear Selected Room',
  props<{ roomId: string }>()
);

/**
 * retrieve list of room information and list of user information per room
 */
export const rooms = createAction(
  '[ucap::chat::room] rooms',
  props<{ localeCode: LocaleCode }>()
);
/**
 * Success of rooms request
 * RoomUserData is detail type.
 */
export const roomsSuccess = createAction(
  '[ucap::chat::room] rooms Success',
  props<{
    roomList: RoomInfo[];
    roomUserInfoMap: {
      [param: string]: UserInfo[];
    };
    syncDate: string;
  }>()
);
/**
 * Failure of rooms request
 */
export const roomsFailure = createAction(
  '[ucap::chat::room] rooms Failure',
  props<{ error: any }>()
);

/**
 * Success of rooms2 request
 * RoomUserData is detail, short type both.
 */
export const rooms2Success = createAction(
  '[ucap::chat::room] rooms2 Success',
  props<{
    roomList: RoomInfo[];
    roomUserInfoMap: {
      [param: string]: {
        userInfoShortList: UserInfoShort[];
        userInfoList: UserInfo[];
      };
    };
    syncDate: string;
  }>()
);
/**
 * Failure of rooms2 request
 */
export const rooms2Failure = createAction(
  '[ucap::chat::room] rooms2 Failure',
  props<{ error: any }>()
);

/**
 * retrieve room information and list of user information
 */
export const room = createAction(
  '[ucap::chat::room] room',
  props<{ req: RoomRequest }>()
);
/**
 * Success of room request
 */
export const roomSuccess = createAction(
  '[ucap::chat::room] room Success',
  props<{
    roomInfo: RoomInfo;
    userInfoList: UserInfo[];
  }>()
);
/**
 * Failure of room request
 */
export const roomFailure = createAction(
  '[ucap::chat::room] room Failure',
  props<{ error: any }>()
);
/**
 * Success of rooms2 request
 * RoomUserData is detail, short type both.
 */
export const room2Success = createAction(
  '[ucap::chat::room] room2 Success',
  props<{
    roomInfo: RoomInfo;
    roomUserInfo: {
      userInfoShortList: UserInfoShort[];
      userInfoList: UserInfo[];
    };
  }>()
);

/**
 * create room
 */
export const create = createAction(
  '[ucap::chat::room] create',
  props<{ req: CreateRequest }>()
);
/**
 * create timer room
 */
export const createTimer = createAction(
  '[ucap::chat::room] createTimer',
  props<{ req: CreateTimerRequest }>()
);
/**
 * Success of create room / timer room request
 */
export const createRoomSuccess = createAction(
  '[ucap::chat::room] create room Success',
  props<{ res: CreateResponse; isForward?: boolean }>()
);
/**
 * Failure of create room / timer room request
 */
export const createRoomFailure = createAction(
  '[ucap::chat::room] create room Failure',
  props<{ error: any }>()
);

/**
 * exit from room
 */
export const del = createAction(
  '[ucap::chat::room] del',
  props<{ req: DeleteRequest }>()
);
/**
 * Success of del request
 */
export const delSuccess = createAction(
  '[ucap::chat::room] del Success',
  props<{ res: DeleteResponse }>()
);
/**
 * Failure of del request
 */
export const delFailure = createAction(
  '[ucap::chat::room] del Failure',
  props<{ error: any }>()
);

/**
 * exit from multi room
 */
export const delMulti = createAction(
  '[ucap::chat::room] delMulti',
  props<{ req: DeleteMultiRequest }>()
);
/**
 * Success of delMulti request
 */
export const delMultiSuccess = createAction(
  '[ucap::chat::room] delMulti Success',
  props<{ res: DeleteMultiResponse }>()
);
/**
 * Failure of delMulti request
 */
export const delMultiFailure = createAction(
  '[ucap::chat::room] delMulti Failure',
  props<{ error: any }>()
);

/**
 * update room information
 */
export const update = createAction(
  '[ucap::chat::room] update',
  props<{ req: UpdateRequest }>()
);
/**
 * Success of update request
 */
export const updateSuccess = createAction(
  '[ucap::chat::room] update Success',
  props<{
    res: UpdateResponse;
  }>()
);
/**
 * Failure of update request
 */
export const updateFailure = createAction(
  '[ucap::chat::room] update Failure',
  props<{ error: any }>()
);
/**
 * Success of update RoomName
 */
export const updateRoomName = createAction(
  '[ucap::chat::room] update Roomname for Notification',
  props<{
    res: UpdateResponse;
  }>()
);

/**
 * update isJoinRoom of user information from true to false
 */
export const excludeUser = createAction(
  '[ucap::chat::room] excludeUser',
  props<{ roomId: string; userSeqs: string[] }>()
);
/**
 * Success of excludeUser request
 */
export const excludeUserSuccess = createAction(
  '[ucap::chat::room] excludeUser Success',
  props<{ roomId: string; userSeqs: string[] }>()
);
/**
 * Failure of excludeUser request
 */
export const excludeUserFailure = createAction(
  '[ucap::chat::room] excludeUser Failure',
  props<{ error: any }>()
);

/**
 * open room
 */
export const open = createAction(
  '[ucap::chat::room] open',
  props<{ roomIds: string[] }>()
);
/**
 * Success of open request
 */
export const openSuccess = createAction(
  '[ucap::chat::room] open Success',
  props<{ roomIds: string[] }>()
);
/**
 * Failure of open request
 */
export const openFailure = createAction(
  '[ucap::chat::room] open Failure',
  props<{ error: any }>()
);

/**
 * close room
 */
export const close = createAction(
  '[ucap::chat::room] close',
  props<{ roomIds: string[] }>()
);
/**
 * Success of close request
 */
export const closeSuccess = createAction(
  '[ucap::chat::room] close Success',
  props<{ roomIds: string[] }>()
);
/**
 * Failure of close request
 */
export const closeFailure = createAction(
  '[ucap::chat::room] close Failure',
  props<{ error: any }>()
);

/**
 * Invite conversation partner to room
 */
export const invite = createAction(
  '[ucap::chat::room] invite',
  props<{ req: InviteRequest; localeCode: LocaleCode }>()
);
/**
 * Success of invite request
 */
export const inviteSuccess = createAction(
  '[ucap::chat::room] invite Success',
  props<{ res: InviteResponse }>()
);
/**
 * Failure of invite request
 */
export const inviteFailure = createAction(
  '[ucap::chat::room] invite Failure',
  props<{ error: any }>()
);

/**
 * Request to ban a conversation partner
 */
export const expel = createAction(
  '[ucap::chat::room] expel',
  props<{ req: ExitForcingRequest }>()
);
/**
 * Success of expel request
 */
export const expelSuccess = createAction(
  '[ucap::chat::room] expel Success',
  props<{ res: ExitForcingResponse }>()
);
/**
 * Failure of expel request
 */
export const expelFailure = createAction(
  '[ucap::chat::room] expel Failure',
  props<{ error: any }>()
);
/**
 * expel notification
 */
export const expelNotification = createAction(
  '[ucap::chat::room] expel Notification',
  props<{ res: ExitForcingResponse }>()
);

/**
 * update interval of timer room
 */
export const updateTimeRoomInterval = createAction(
  '[ucap::chat::room] updateTimeRoomInterval',
  props<{ req: UpdateTimerSetRequest }>()
);
/**
 * Success of updateTimeRoomInterval request
 */
export const updateTimeRoomIntervalSuccess = createAction(
  '[ucap::chat::room] updateTimeRoomInterval Success',
  props<{ res: UpdateTimerSetResponse }>()
);
/**
 * Failure of updateTimeRoomInterval request
 */
export const updateTimeRoomIntervalFailure = createAction(
  '[ucap::chat::room] updateTimeRoomInterval Failure',
  props<{ error: any }>()
);

/**
 * notification for invited users
 */
export const inviteNotification = createAction(
  '[ucap::chat::room] invite Notification',
  props<{ noti: InviteNotification; localeCode: LocaleCode }>()
);

/**
 * notification for exited user
 */
export const exitNotification = createAction(
  '[ucap::chat::room] Exit Notification',
  props<{ roomId: string; senderSeq: string }>()
);

export const updateUnreadCount = createAction(
  '[ucap::chat::room] Update unread count',
  props<{ roomId: string; noReadCnt?: number }>()
);

/**
 * room user info update by Notification
 */
// export const userInfoListUpdate = createAction(
//   '[ucap::chat::room] user info update',
//   props<{
//     roomInfo: RoomInfo;
//     roomUserInfo: {
//       userInfoShortList: UserInfoShort[];
//       userInfoList: RoomUserInfo[];
//     };
//   }>()
// );
