import { createReducer, on } from '@ngrx/store';

import { Info, RoomInfo, EventJson } from '@ucap/domain-chat';

import * as ChattingActions from '../chatting/actions';
import { ChatUtil } from '../../utils/chat.util';

import {
  initialState,
  adapterRoom,
  adapterRoomUser,
  RoomUserMap,
  RoomUserShortMap,
  adapterRoomUserShort
} from './state';
import {
  roomsSuccess,
  roomSuccess,
  excludeUserSuccess,
  delSuccess,
  rooms2Success,
  delMultiSuccess,
  updateSuccess,
  room2Success,
  createRoomSuccess,
  updateUnreadCount,
  updateRoomName,
  updateTimeRoomIntervalSuccess
} from './actions';

export const reducer = createReducer(
  initialState,

  on(roomsSuccess, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const userInfoList = action.roomUserInfoMap[key];
        if (!!userInfoList && 0 < userInfoList.length) {
          roomUserMapList.push({ roomId: key, userInfos: userInfoList });
        }
      }
    }

    let unreadTotal = 0;

    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      const matchedInfo = action.roomList.find(
        (item) => item.roomId === value.roomId
      );
      if (!!matchedInfo) {
        unreadTotal += isNaN(matchedInfo.noReadCnt) ? 0 : matchedInfo.noReadCnt;
      } else {
        unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
      }
    }

    /**
     * Correcting expiredFileStdSeq.
     * 대화방이 열린 상태에서 새로고침 할 때 'RoomInfo' 보다 'SyncRoomInfo'가 나중에 유입되어 expiredFileStdSeq 값을 0으로 덮어쓰는 문제로 보정코드 삽입.
     * 'SyncRoomInfo' 에서는 expiredFileStdSeq 값이 0으로 고정되어 내려온다.(DB 부하 문제)
     */
    const trgtRoomList = action.roomList.map((roomInfo) => {
      let rtnValue: RoomInfo = roomInfo;

      // tslint:disable-next-line: forin
      for (const key in state.rooms.entities) {
        const value = state.rooms.entities[key];
        if (value.roomId === roomInfo.roomId && value.expiredFileStdSeq > 0) {
          rtnValue = {
            ...rtnValue,
            expiredFileStdSeq: value.expiredFileStdSeq
          };
          break;
        }
      }

      return rtnValue;
    });

    return {
      ...state,
      rooms: adapterRoom.upsertMany(trgtRoomList, {
        ...state.rooms,
        syncDate: action.syncDate
      }),
      roomUsers: adapterRoomUser.upsertMany(roomUserMapList, {
        ...state.roomUsers
      }),
      unreadTotal
    };
  }),

  on(rooms2Success, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];
    const roomUserShortMapList: RoomUserShortMap[] = [];

    for (const key in action.roomUserInfoMap) {
      if (action.roomUserInfoMap.hasOwnProperty(key)) {
        const userInfosMap = action.roomUserInfoMap[key];
        if (!!userInfosMap && 0 < userInfosMap.userInfoList.length) {
          roomUserMapList.push({
            roomId: key,
            userInfos: userInfosMap.userInfoList
          });
        }
        if (!!userInfosMap && 0 < userInfosMap.userInfoShortList.length) {
          roomUserShortMapList.push({
            roomId: key,
            userInfos: userInfosMap.userInfoShortList
          });
        }
      }
    }

    let unreadTotal = 0;

    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      const matchedInfo = action.roomList.find(
        (item) => item.roomId === value.roomId
      );
      if (!!matchedInfo) {
        unreadTotal += isNaN(matchedInfo.noReadCnt) ? 0 : matchedInfo.noReadCnt;
      } else {
        unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
      }
    }

    /**
     * Correcting expiredFileStdSeq.
     * 대화방이 열린 상태에서 새로고침 할 때 'RoomInfo' 보다 'SyncRoomInfo'가 나중에 유입되어 expiredFileStdSeq 값을 0으로 덮어쓰는 문제로 보정코드 삽입.
     * 'SyncRoomInfo' 에서는 expiredFileStdSeq 값이 0으로 고정되어 내려온다.(DB 부하 문제)
     */
    const trgtRoomList = action.roomList.map((roomInfo) => {
      let rtnValue: RoomInfo = roomInfo;

      // tslint:disable-next-line: forin
      for (const key in state.rooms.entities) {
        const value = state.rooms.entities[key];
        if (value.roomId === roomInfo.roomId && value.expiredFileStdSeq > 0) {
          rtnValue = {
            ...rtnValue,
            expiredFileStdSeq: value.expiredFileStdSeq
          };
          break;
        }
      }

      return rtnValue;
    });

    return {
      ...state,
      rooms: adapterRoom.upsertMany(trgtRoomList, {
        ...state.rooms,
        syncDate: action.syncDate
      }),
      roomUsers: adapterRoomUser.upsertMany(roomUserMapList, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.upsertMany(roomUserShortMapList, {
        ...state.roomUsersShort
      }),
      unreadTotal
    };
  }),

  on(roomSuccess, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];

    if (!!action.userInfoList && 0 < action.userInfoList.length) {
      roomUserMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.userInfoList
      });
    }

    return {
      ...state,
      rooms: adapterRoom.upsertOne(action.roomInfo, {
        ...state.rooms
      }),
      roomUsers:
        0 < roomUserMapList.length
          ? adapterRoomUser.upsertMany(roomUserMapList, {
              ...state.roomUsers
            })
          : state.roomUsers
    };
  }),

  on(room2Success, (state, action) => {
    const roomUserMapList: RoomUserMap[] = [];
    const roomUserShortMapList: RoomUserShortMap[] = [];

    if (
      !!action.roomUserInfo.userInfoList &&
      0 < action.roomUserInfo.userInfoList.length
    ) {
      roomUserMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.roomUserInfo.userInfoList
      });
    }
    if (
      !!action.roomUserInfo.userInfoShortList &&
      0 < action.roomUserInfo.userInfoShortList.length
    ) {
      roomUserShortMapList.push({
        roomId: action.roomInfo.roomId,
        userInfos: action.roomUserInfo.userInfoShortList
      });
    }

    return {
      ...state,
      rooms: adapterRoom.upsertOne(action.roomInfo, {
        ...state.rooms
      }),
      roomUsers:
        0 < roomUserMapList.length
          ? adapterRoomUser.upsertMany(roomUserMapList, {
              ...state.roomUsers
            })
          : state.roomUsers,
      roomUsersShort:
        0 < roomUserShortMapList.length
          ? adapterRoomUserShort.upsertMany(roomUserShortMapList, {
              ...state.roomUsersShort
            })
          : state.roomUsersShort
    };
  }),

  on(excludeUserSuccess, (state, action) => {
    const roomId = action.roomId;

    let roomUser: RoomUserMap = state.roomUsers.entities[roomId];
    if (!!roomUser) {
      roomUser = {
        ...roomUser,
        userInfos: state.roomUsers.entities[roomId].userInfos.map(
          (roomUserInfo) => {
            if (
              action.userSeqs.some((seq) => seq + '' === roomUserInfo.seq + '')
            ) {
              return {
                ...roomUserInfo,
                isJoinRoom: false
              };
            } else {
              return roomUserInfo;
            }
          }
        )
      };
    }
    let roomUserShort: RoomUserShortMap = state.roomUsersShort.entities[roomId];
    if (!!roomUserShort) {
      roomUserShort = {
        ...roomUserShort,
        userInfos: state.roomUsersShort.entities[roomId].userInfos.map(
          (roomUserInfo) => {
            if (
              action.userSeqs.some((seq) => seq + '' === roomUserInfo.seq + '')
            ) {
              return {
                ...roomUserInfo,
                isJoinRoom: false
              };
            } else {
              return roomUserInfo;
            }
          }
        )
      };
    }

    const currentRoomInfo = state.rooms.entities[roomId];
    let roomInfo: RoomInfo;
    if (!!currentRoomInfo) {
      roomInfo = {
        ...currentRoomInfo,
        joinUserCount: !!roomUser
          ? roomUser.userInfos.filter((item) => !!item.isJoinRoom).length
          : !!roomUserShort
          ? roomUserShort.userInfos.filter((item) => !!item.isJoinRoom).length
          : currentRoomInfo.joinUserCount
      };
    }

    return {
      ...state,
      rooms: !!roomInfo
        ? adapterRoom.upsertOne(roomInfo, { ...state.rooms })
        : state.rooms,
      roomUsers: !!roomUser
        ? adapterRoomUser.upsertOne(roomUser, {
            ...state.roomUsers
          })
        : state.roomUsers,
      roomUsersShort: !!roomUserShort
        ? adapterRoomUserShort.upsertOne(roomUserShort, {
            ...state.roomUsersShort
          })
        : state.roomUsersShort
    };
  }),

  on(createRoomSuccess, (state, action) => {
    const standby = state.standbyRooms;
    const curRoomId = action.res.roomId;

    if (
      !!action.isForward ||
      standby.findIndex((roomId) => roomId === curRoomId) > -1 ||
      !!state.rooms.entities[curRoomId]
    ) {
      // Exist.
      return state;
    } else {
      // Not Exist.
      return {
        ...state,
        standbyRooms: [...state.standbyRooms, curRoomId]
      };
    }
  }),

  on(updateSuccess, (state, action) => {
    const roomInfo = {
      ...state.rooms.entities[action.res.roomId],
      roomName: action.res.roomName,
      receiveAlarm: action.res.receiveAlarm
    } as RoomInfo;

    return {
      ...state,
      rooms: adapterRoom.upsertOne(roomInfo, { ...state.rooms })
    };
  }),

  on(updateTimeRoomIntervalSuccess, (state, action) => {
    const roomInfo = {
      ...state.rooms.entities[action.res.roomId],
      timeRoomInterval: action.res.timerInterval
    } as RoomInfo;

    return {
      ...state,
      rooms: adapterRoom.upsertOne(roomInfo, { ...state.rooms })
    };
  }),

  on(updateRoomName, (state, action) => {
    const roomInfo = {
      ...state.rooms.entities[action.res.roomId],
      roomName: action.res.roomName
    } as RoomInfo;

    return {
      ...state,
      rooms: adapterRoom.upsertOne(roomInfo, { ...state.rooms })
    };
  }),

  on(delSuccess, (state, action) => {
    const roomId = action.res.roomId;
    const room = state.rooms.entities[roomId];

    if (!room) {
      return state;
    }

    return {
      ...state,
      rooms: adapterRoom.removeOne(roomId, { ...state.rooms }),
      roomUsers: adapterRoomUser.removeOne(roomId, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.removeOne(roomId, {
        ...state.roomUsersShort
      })
    };
  }),

  on(delMultiSuccess, (state, action) => {
    const roomIds: string[] = action.res.roomIds;

    if (!roomIds || roomIds.length === 0) {
      return state;
    }

    return {
      ...state,
      rooms: adapterRoom.removeMany(roomIds, { ...state.rooms }),
      roomUsers: adapterRoomUser.removeMany(roomIds, {
        ...state.roomUsers
      }),
      roomUsersShort: adapterRoomUserShort.removeMany(roomIds, {
        ...state.roomUsersShort
      })
    };
  }),

  on(updateUnreadCount, (state, action) => {
    const roomId = action.roomId;
    const noReadCnt = action.noReadCnt || 0;
    const roomInfo = state.rooms.entities[roomId];

    let unreadTotal = 0;
    // tslint:disable-next-line: forin
    for (const key in state.rooms.entities) {
      const value = state.rooms.entities[key];
      unreadTotal += isNaN(value.noReadCnt) ? 0 : value.noReadCnt;
    }

    if (!!roomInfo) {
      return {
        ...state,
        rooms: adapterRoom.updateOne(
          { id: roomId, changes: { ...roomInfo, noReadCnt } },
          { ...state.rooms }
        ),
        unreadTotal
      };
    } else {
      return state;
    }
  }),

  /*******************************************************************
   * [Chatting Action watching.]
   *******************************************************************/
  on(ChattingActions.readSuccess, (state, action) => {
    const roomId = action.roomId;
    const trgtUserSeq = action.SENDER_SEQ;

    if (!roomId || !trgtUserSeq) {
      return state;
    }

    let roomUser: RoomUserMap = state.roomUsers.entities[roomId];
    if (!!roomUser) {
      roomUser = {
        ...roomUser,
        userInfos: state.roomUsers.entities[roomId].userInfos.map(
          (roomUserInfo) => {
            if (roomUserInfo.seq === Number(trgtUserSeq)) {
              return {
                ...roomUserInfo,
                lastReadEventSeq: action.lastReadSeq
              };
            } else {
              return roomUserInfo;
            }
          }
        )
      };
    }
    let roomUserShort: RoomUserShortMap = state.roomUsersShort.entities[roomId];
    if (!!roomUserShort) {
      roomUserShort = {
        ...roomUserShort,
        userInfos: state.roomUsersShort.entities[roomId].userInfos.map(
          (roomUserInfo) => {
            if (roomUserInfo.seq === Number(trgtUserSeq)) {
              return {
                ...roomUserInfo,
                lastReadEventSeq: action.lastReadSeq
              };
            } else {
              return roomUserInfo;
            }
          }
        )
      };
    }

    return {
      ...state,
      roomUsers: !!roomUser
        ? adapterRoomUser.upsertOne(roomUser, {
            ...state.roomUsers
          })
        : state.roomUsers,
      roomUsersShort: !!roomUserShort
        ? adapterRoomUserShort.upsertOne(roomUserShort, {
            ...state.roomUsersShort
          })
        : state.roomUsersShort
    };
  }),

  on(ChattingActions.addEventSuccess, (state, action) => {
    const roomId = action.roomId;
    const info: Info<EventJson> = action.info;

    if (!roomId || !info) {
      return state;
    }

    const currentRoomInfo = state.rooms.entities[roomId];
    const fixedStandByRooms = state.standbyRooms.filter(
      (standbyRoomId) => standbyRoomId !== roomId
    );

    if (!!currentRoomInfo) {
      const finalEventMessage:
        | string
        | null = ChatUtil.convertFinalEventMessage(
        action.info.type,
        action.info.sentMessageJson || action.info.sentMessage
      );

      if (!finalEventMessage) {
        /**
         * 해당 타입은 메시지를 갱신하지 않는다.
         */
        return state;
      } else {
        const roomInfo = {
          ...currentRoomInfo,
          finalEventType: info.type,
          finalEventDate: info.sendDate,
          finalEventMessage,
          finalEventSeq: info.seq
        } as RoomInfo;

        return {
          ...state,
          rooms: adapterRoom.upsertOne(roomInfo, { ...state.rooms }),
          standbyRooms: fixedStandByRooms
        };
      }
    } else {
      return state;
    }
  })
);
