import { createReducer, on } from '@ngrx/store';

import { UserInfoUpdateType, UserInfoF } from '@ucap/domain-organization';

import { StringUtil } from '@ucap/ng-core';

import { initialState, adapterBuddy } from './state';
import {
  buddy2Success,
  delSuccess,
  updateSuccess,
  nicknameSuccess,
  buddyInfoUpdate
} from './actions';

export const reducer = createReducer(
  initialState,
  on(buddy2Success, (state, action) => {
    return {
      ...state,
      buddies: adapterBuddy.upsertMany(action.buddyList, {
        ...state.buddies,
        syncDate: action.syncDate
      })
    };
  }),

  on(delSuccess, (state, action) => {
    const res = action.res;
    return {
      ...state,
      buddies: adapterBuddy.removeMany(res.userSeqs, {
        ...state.buddies
      })
    };
  }),

  on(updateSuccess, (state, action) => {
    const res = action.res;
    const userInfo: UserInfoF = {
      ...state.buddies.entities[res.seq],
      isFavorit: res.isFavorit
    };

    return {
      ...state,
      buddies: adapterBuddy.upsertOne(userInfo, { ...state.buddies })
    };
  }),

  on(nicknameSuccess, (state, action) => {
    const res = action.res;
    const userInfo: UserInfoF = {
      ...state.buddies.entities[res.userSeq],
      nickName: res.nickname
    };

    return {
      ...state,
      buddies: adapterBuddy.upsertOne(userInfo, { ...state.buddies })
    };
  }),

  on(buddyInfoUpdate, (state, action) => {
    const noti = action.noti;
    let buddyInfo: UserInfoF;

    switch (noti.type) {
      case UserInfoUpdateType.Image:
        {
          const imgInfo = StringUtil.getProfileSubDir(noti.info);

          buddyInfo = {
            ...state.buddies.entities[noti.SENDER_SEQ],
            profileImageFile: imgInfo
          };
        }
        break;
      case UserInfoUpdateType.Intro:
        {
          buddyInfo = {
            ...state.buddies.entities[noti.SENDER_SEQ],
            intro: noti.info
          };
        }
        break;
      case UserInfoUpdateType.TelephoneVisible:
        {
          buddyInfo = {
            ...state.buddies.entities[noti.SENDER_SEQ]
          };
        }
        break;
    }

    return {
      ...state,
      buddies: adapterBuddy.upsertOne(buddyInfo, { ...state.buddies })
    };
  })
);
