import { withLatestFrom, tap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { select, Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import { modifyInfoSuccess } from '../user/actions';
import { UserSelector } from '../state';

import { userNotification } from './actions';

@Injectable()
export class Effects {
  userNotification$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(userNotification),
        withLatestFrom(this.store.pipe(select(UserSelector.user))),
        tap(([action, user]) => {
          if (Number(action.noti.SENDER_SEQ) === Number(user.info.seq)) {
            // my
            const noti = action.noti;

            this.store.dispatch(
              modifyInfoSuccess({
                res: {
                  SENDER_SEQ: noti.SENDER_SEQ,
                  info: noti.info,
                  type: noti.type
                }
              })
            );
          }
        })
      );
    },
    { dispatch: false }
  );

  constructor(private actions$: Actions, private store: Store<any>) {}
}
