import { of } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import { ExternalApiService } from '@ucap/ng-api-external';

import { companies, companiesSuccess, companiesFailure } from './actions';

@Injectable()
export class Effects {
  companies$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(companies),
      map(action => action.req),
      switchMap(req =>
        this.externalApiService.companyList(req).pipe(
          map(res => companiesSuccess({ res })),
          catchError(error => of(companiesFailure({ error })))
        )
      )
    );
  });

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private externalApiService: ExternalApiService
  ) {}
}
