import { of } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import { DeptDivisionCode } from '@ucap/protocol-query';

import { QueryProtocolService } from '@ucap/ng-protocol-query';

import { init as userInit } from '../user/actions';

import {
  dept,
  deptSuccess,
  deptFailure,
  myDeptUserSuccess,
  myDeptUser,
  myDeptUserFailure
} from './actions';

@Injectable()
export class Effects {
  sessionCreatedForGroups$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(userInit),
      map((action) =>
        dept({
          req: {
            divCd: DeptDivisionCode.Organization,
            companyCode: action.user.companyCode
          }
        })
      )
    );
  });

  dept$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(dept),
      map((action) => action.req),
      switchMap((req) => {
        return this.queryProtocolService.dept(req).pipe(
          map((res) => {
            return deptSuccess({ departmentInfoList: res.departmentInfoList });
          }),
          catchError((error) => of(deptFailure({ error })))
        );
      })
    );
  });

  myDeptUser$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(myDeptUser),
      map((action) => action.req),
      switchMap((req) => {
        return this.queryProtocolService.deptUser(req).pipe(
          map((res) => {
            return myDeptUserSuccess({ userInfos: res.userInfos });
          }),
          catchError((error) => of(myDeptUserFailure({ error })))
        );
      })
    );
  });

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private queryProtocolService: QueryProtocolService
  ) {}
}
