import { of } from 'rxjs';
import { map, exhaustMap, catchError, concatMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Store } from '@ngrx/store';

import { StatusProtocolService } from '@ucap/ng-protocol-status';

import {
  status,
  statusSuccess,
  statusFailure,
  bulkInfo,
  bulkInfoSuccess,
  bulkInfoFailure,
  messageUpdate,
  messageUpdateSuccess,
  messageUpdateFailure
} from './actions';

@Injectable()
export class Effects {
  bulkInfo$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(bulkInfo),
        concatMap((req) => {
          return this.statusProtocolService.bulkInfo(req).pipe(
            map((res) => {
              this.store.dispatch(
                bulkInfoSuccess({
                  statusBulkInfoList: res.statusBulkInfoList
                })
              );
            }),
            catchError((error) => of(bulkInfoFailure({ error })))
          );
        })
      );
    },
    { dispatch: false }
  );

  status$ = createEffect(() =>
    this.actions$.pipe(
      ofType(status),
      map((action) => action.req),
      exhaustMap((req) => {
        return this.statusProtocolService.status(req).pipe(
          map((res) => {
            return statusSuccess({ res });
          }),
          catchError((error) => of(statusFailure({ error })))
        );
      })
    )
  );

  messageUpdate$ = createEffect(() =>
    this.actions$.pipe(
      ofType(messageUpdate),
      map((action) => action.req),
      exhaustMap((req) => {
        return this.statusProtocolService.messageUpdate(req).pipe(
          map((res) => {
            return messageUpdateSuccess({ res });
          }),
          catchError((error) => of(messageUpdateFailure({ error })))
        );
      })
    )
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private statusProtocolService: StatusProtocolService
  ) {}
}
