import { createReducer, on } from '@ngrx/store';

import {
  StatusBulkInfo,
  TerminalStatusInfo,
  TerminalStatusNumber
} from '@ucap/domain-status';

import { initialState, State, adapterStatusBulkInfo } from './state';
import {
  bulkInfoSuccess,
  statusNotification,
  statusSuccess,
  bulkInfo
} from './actions';

export const reducer = createReducer(
  initialState,
  on(bulkInfo, (state, action) => {
    return {
      ...state,
      statusBulkInfoProcessing: true
    } as State;
  }),

  on(bulkInfoSuccess, (state, action) => {
    return {
      ...state,
      statusBulkInfo: adapterStatusBulkInfo.upsertMany(
        action.statusBulkInfoList,
        {
          ...state.statusBulkInfo
        }
      ),
      statusBulkInfoProcessing: false
    } as State;
  }),

  on(statusNotification, (state, action) => {
    const noti = action.noti;

    const statusBulkInfoState: StatusBulkInfo = {
      userSeq: noti.userSeq,
      conferenceStatus: noti.conferenceStatus,
      imessengerStatus: noti.imessengerStatus,
      mobileConferenceStatus: noti.mobileConferenceStatus,
      mobileStatus: noti.mobileStatus,
      pcStatus: noti.pcStatus,
      phoneStatus: noti.phoneStatus,
      statusMessage: noti.statusMessage,
      terminalStatus: TerminalStatusInfo.Unknown,
      terminalStatusNumber: TerminalStatusNumber.Unknown,
      workstatus: noti.workstatus
    };

    return {
      ...state,
      statusBulkInfo: adapterStatusBulkInfo.upsertOne(statusBulkInfoState, {
        ...state.statusBulkInfo
      }),
      statusBulkInfoProcessing: false
    };
  }),

  on(statusSuccess, (state, action) => {
    const statusBulkInfoState: StatusBulkInfo = {
      ...state.statusBulkInfo.entities[action.res.SENDER_SEQ],
      pcStatus: action.res.statusType,
      statusMessage: action.res.statusMessage
    };

    return {
      ...state,
      statusBulkInfo: adapterStatusBulkInfo.updateOne(
        { id: action.res.SENDER_SEQ, changes: statusBulkInfoState },
        { ...state.statusBulkInfo }
      ),
      statusBulkInfoProcessing: false
    } as State;
  })
);
