import { of } from 'rxjs';
import { map, exhaustMap, catchError } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { UserResponse } from '@ucap/protocol-info';

import { InfoProtocolService } from '@ucap/ng-protocol-info';

import { modifyInfo, modifyInfoSuccess, modifyInfoFailure } from './actions';

@Injectable()
export class Effects {
  infoUser$ = createEffect(() =>
    this.actions$.pipe(
      ofType(modifyInfo),
      map((action) => action.req),
      exhaustMap((req) =>
        this.infoProtocolService.user(req).pipe(
          map((res: UserResponse) => {
            return modifyInfoSuccess({
              res
            });
          }),
          catchError((error) => of(modifyInfoFailure({ error })))
        )
      )
    )
  );

  constructor(
    private actions$: Actions,
    private infoProtocolService: InfoProtocolService
  ) {}
}
