import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ReactiveFormsModule } from '@angular/forms';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatButtonModule } from '@angular/material/button';
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatSelectModule } from '@angular/material/select';

import { I18nModule, UCAP_I18N_NAMESPACE } from '@ucap/ng-i18n';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import { ChangePasswordComponent } from './components/change-password.component';
import { LoginComponent } from './components/login.component';

const COMPONENTS = [ChangePasswordComponent, LoginComponent];
const DIALOGS = [];
const PIPES = [];
const DIRECTIVES = [];
const SERVICES = [];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class AuthenticationUiRootModule {}

@NgModule({
  imports: [
    CommonModule,
    ReactiveFormsModule,

    FlexLayoutModule,

    MatButtonModule,
    MatCheckboxModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatProgressSpinnerModule,
    MatSelectModule,

    I18nModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS],
  providers: [
    {
      provide: UCAP_I18N_NAMESPACE,
      useValue: ['authentication']
    }
  ]
})
export class AuthenticationUiModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<AuthenticationUiRootModule> {
    return {
      ngModule: AuthenticationUiRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
