import { moduleMetadata } from '@storybook/angular';
import { action } from '@storybook/addon-actions';
import { linkTo } from '@storybook/addon-links';

import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

import { Company } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';
import { I18nService, UCAP_I18N_NAMESPACE } from '@ucap/ng-i18n';

import { AuthenticationUiModule } from '../authentication-ui.module';

import { LoginComponent } from './login.component';

const actionsData = {
  login: action('login')
};

const inputData = {
  companyList: [
    { companyName: 'LG CNS', companyCode: 'GUC100' },
    { companyName: 'LG UCAP', companyCode: 'GUC101' }
  ] as Company[]
};

export default {
  title: 'ui-authentication::LoginComponent',
  component: LoginComponent,
  decorators: [
    moduleMetadata({
      imports: [BrowserModule, BrowserAnimationsModule, AuthenticationUiModule],
      providers: [
        { provide: LogService, useValue: new LogService({}) },
        { provide: I18nService, useClass: I18nService },
        {
          provide: UCAP_I18N_NAMESPACE,
          useValue: 'authentication'
        }
      ]
    })
  ]
};

export const Default = () => ({
  component: LoginComponent,
  props: {
    companyList: inputData.companyList,
    login: actionsData.login
  }
});

export const InputContents = () => ({
  component: LoginComponent,
  props: {
    companyList: inputData.companyList,
    companyCode: 'GUC100',
    loginId: 'test-loginid',
    login: actionsData.login
  }
});
