import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { FlexLayoutModule } from '@angular/flex-layout';

import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatRippleModule } from '@angular/material/core';
import { MatTreeModule } from '@angular/material/tree';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatGridListModule } from '@angular/material/grid-list';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';

import { PerfectScrollbarModule } from 'ngx-perfect-scrollbar';

import { UiDateModule } from '@ucap/ng-ui/date';
import { UiPhoneModule } from '@ucap/ng-ui/phone';
import { UiscrollingModule } from '@ucap/ng-ui/scrolling';

import { I18nModule, UCAP_I18N_NAMESPACE } from '@ucap/ng-i18n';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import {
  HistoryExpansionComponent,
  HistoryExpansionHeaderDirective,
  HistoryExpansionNodeDirective
} from './components/history-expansion.component';
import { HistoryListItemComponent } from './components/history-list-item.component';
import {
  CallTimelineItemListComponent,
  CallTimelineItemListNodeDirective
} from './components/timeline-item-list.component';
import {
  CallTimelineItemList01Component,
  CallTimelineItemList01NodeDirective
} from './components/timeline-item-list-01.component';
import { TimelineItemComponent } from './components/timeline-item.component';
import { DialpadComponent } from './components/dialpad.component';

const COMPONENTS = [
  DialpadComponent,
  HistoryExpansionComponent,
  HistoryListItemComponent,
  CallTimelineItemListComponent,
  CallTimelineItemList01Component,
  TimelineItemComponent
];
const DIALOGS = [];
const PIPES = [];
const DIRECTIVES = [
  CallTimelineItemListNodeDirective,
  CallTimelineItemList01NodeDirective,
  HistoryExpansionHeaderDirective,
  HistoryExpansionNodeDirective
];
const SERVICES = [];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class CallUiRootModule {}

@NgModule({
  imports: [
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    FlexLayoutModule,

    MatButtonModule,
    MatIconModule,
    MatRippleModule,
    MatTreeModule,
    MatProgressBarModule,
    MatTooltipModule,
    MatFormFieldModule,
    MatInputModule,
    MatGridListModule,

    PerfectScrollbarModule,

    I18nModule,

    UiDateModule,
    UiPhoneModule,
    UiscrollingModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS],
  providers: [
    {
      provide: UCAP_I18N_NAMESPACE,
      useValue: ['call', 'common']
    }
  ]
})
export class CallUiModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<CallUiRootModule> {
    return {
      ngModule: CallUiRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
