import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Directive,
  ContentChild,
  TemplateRef,
  ViewChild,
  ElementRef
} from '@angular/core';

import {
  VirtualScrollViewportComponent,
  MeasureSizeVirtualScrollDirective
} from '@ucap/ng-ui/scrolling';
import { CallHistory } from '@ucap/domain-call';
import { PerfectScrollbarDirective } from 'ngx-perfect-scrollbar';

@Directive({
  selector: '[ucapCallTimelineItemList01Node]'
})
export class CallTimelineItemList01NodeDirective {}

@Component({
  selector: 'ucap-call-timeline-item-list-01',
  templateUrl: './timeline-item-list-01.component.html',
  styleUrls: ['./timeline-item-list-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CallTimelineItemList01Component implements OnInit, OnDestroy {
  @Input()
  itemInfos: CallHistory[];

  @Input()
  maxBufferPx: number;
  @Input()
  minBufferPx: number;

  @ContentChild(CallTimelineItemList01NodeDirective, {
    read: TemplateRef,
    static: false
  })
  nodeTemplate: TemplateRef<CallTimelineItemList01NodeDirective>;

  @ViewChild('vsList', { static: false })
  vsList: VirtualScrollViewportComponent;

  @ViewChild('vsList', {
    static: true,
    read: MeasureSizeVirtualScrollDirective
  })
  measureSize: MeasureSizeVirtualScrollDirective;

  @ViewChild(PerfectScrollbarDirective, {
    static: true,
    read: PerfectScrollbarDirective
  })
  psDirectiveRef: PerfectScrollbarDirective;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  /** About scrolling */
  keyOf = (item: CallHistory): number => {
    return item.callLogSeq;
  };

  onMeasured() {}

  onContentSizeChanged(size: number) {
    if (!!this.psDirectiveRef) {
      this.psDirectiveRef.update();
    }
  }
}
