import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Directive,
  ContentChild,
  TemplateRef,
  ViewChild,
  ElementRef
} from '@angular/core';

import { VirtualScrollViewportComponent } from '@ucap/ng-ui/scrolling';
import { CallHistory } from '@ucap/domain-call';

@Directive({
  selector: '[ucapCallTimelineItemListNode]'
})
export class CallTimelineItemListNodeDirective {}

@Component({
  selector: 'ucap-call-timeline-item-list',
  templateUrl: './timeline-item-list.component.html',
  styleUrls: ['./timeline-item-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CallTimelineItemListComponent implements OnInit, OnDestroy {
  @Input()
  itemInfos: CallHistory[];

  @Input()
  vsItemSize: number;

  @ContentChild(CallTimelineItemListNodeDirective, {
    read: TemplateRef,
    static: false
  })
  nodeTemplate: TemplateRef<CallTimelineItemListNodeDirective>;

  @ViewChild('container', { static: true })
  container: ElementRef<HTMLElement>;

  @ViewChild('cvsvList', { static: true })
  cvsvList: VirtualScrollViewportComponent;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }
}
