import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatBadgeModule } from '@angular/material/badge';
import { MatButtonModule } from '@angular/material/button';
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatIconModule } from '@angular/material/icon';
import { MatRippleModule } from '@angular/material/core';
import { MatTreeModule } from '@angular/material/tree';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatMenuModule } from '@angular/material/menu';

import { PerfectScrollbarModule } from 'ngx-perfect-scrollbar';

import { UiCoreModule } from '@ucap/ng-ui/core';
import { UiDateModule } from '@ucap/ng-ui/date';
import { UiscrollingModule } from '@ucap/ng-ui/scrolling';

import { OrganizationUiModule } from '@ucap/ng-ui-organization';

import { I18nModule, UCAP_I18N_NAMESPACE } from '@ucap/ng-i18n';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import {
  ImageListComponent,
  ImageListNodeDirective
} from './components/image-list.component';
import {
  FileListComponent,
  FileListNodeDirective
} from './components/file-list.component';
import { ImageListItem01Component } from './components/image-list-item-01.component';
import { FileListItem01Component } from './components/file-list-item-01.component';
import {
  RoomExpansionComponent,
  RoomExpansionHeaderDirective,
  RoomExpansionNodeDirective
} from './components/room-expansion.component';
import { RoomListItem01Component } from './components/room-list-item-01.component';
import { InformationComponent } from './components/message-box/information.component';
import { TextComponent } from './components/message-box/text.component';
import { FileComponent } from './components/message-box/file.component';
import { AttachFileComponent } from './components/message-box/attach-file.component';
import { ImageComponent } from './components/message-box/image.component';
import { VideoComponent } from './components/message-box/video.component';
import { StickerComponent } from './components/message-box/sticker.component';
import { MassComponent } from './components/message-box/mass.component';
import { ScheduleComponent } from './components/message-box/schedule.component';
import { VideoConferenceComponent } from './components/message-box/video-conference.component';
import { TranslationComponent } from './components/message-box/translation.component';
import { MassTranslationComponent } from './components/message-box/mass-translation.component';
import { RecallComponent } from './components/message-box/recall.component';
import { ReadHereComponent } from './components/message-box/read-here.component';
import { DateSplitterComponent } from './components/message-box/date-splitter.component';
import { SmsComponent } from './components/message-box/sms.component';
import { ReplyComponent } from './components/message-box/reply.component';
import {
  ChatItemListComponent,
  ChatItemListNodeDirective
} from './components/item-list.component';

const COMPONENTS = [
  ImageListItem01Component,
  FileListItem01Component,
  FileListComponent,
  ImageListComponent,
  RoomExpansionComponent,
  RoomListItem01Component,
  ChatItemListComponent,

  InformationComponent,
  TextComponent,
  FileComponent,
  AttachFileComponent,
  ImageComponent,
  VideoComponent,
  StickerComponent,
  MassComponent,
  ScheduleComponent,
  VideoConferenceComponent,
  TranslationComponent,
  MassTranslationComponent,
  RecallComponent,

  ReadHereComponent,
  DateSplitterComponent,

  SmsComponent,
  ReplyComponent
];
const DIALOGS = [];
const PIPES = [];
const DIRECTIVES = [
  RoomExpansionHeaderDirective,
  RoomExpansionNodeDirective,
  FileListNodeDirective,
  ImageListNodeDirective,
  ChatItemListNodeDirective
];
const SERVICES = [];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class ChatUiRootModule {}

@NgModule({
  imports: [
    CommonModule,
    FlexLayoutModule,

    MatBadgeModule,
    MatButtonModule,
    MatCheckboxModule,
    MatIconModule,
    MatRippleModule,
    MatTreeModule,
    MatMenuModule,
    MatProgressBarModule,
    MatTooltipModule,

    PerfectScrollbarModule,

    I18nModule,

    UiCoreModule,
    UiDateModule,
    UiscrollingModule,

    OrganizationUiModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS],
  providers: [
    {
      provide: UCAP_I18N_NAMESPACE,
      useValue: ['chat', 'common']
    }
  ]
})
export class ChatUiModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<ChatUiRootModule> {
    return {
      ngModule: ChatUiRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
