import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output
} from '@angular/core';

import { FileDownloadItem } from '@ucap/domain-common';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  FileInfo
} from '@ucap/domain-chat';

import { DateOptions } from '@ucap/ng-ui/date';

@Component({
  selector: 'ucap-chat-file-list-item-01',
  templateUrl: './file-list-item-01.component.html',
  styleUrls: ['./file-list-item-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class FileListItem01Component implements OnInit, OnDestroy {
  @Input()
  fileInfo: FileInfo;

  @Input()
  senderInfo: RoomUserInfo | RoomUserInfoShort;

  @Input()
  downloadTotalCount: number;
  @Input()
  downloadedCount: number;

  @Input()
  fileRetentionPeriod: number;

  @Input()
  expired = false;

  @Input()
  checked = false;

  @Input()
  checkable = true;

  @Input()
  fileDownloadItem: FileDownloadItem;

  @Output()
  changeCheck = new EventEmitter<{
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }>();

  @Output()
  showDownCheck = new EventEmitter<FileInfo>();

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickShowDownCheck() {
    this.showDownCheck.emit(this.fileInfo);
  }

  onChangeCheck(checked: boolean) {
    this.changeCheck.emit({
      checked,
      fileInfo: this.fileInfo,
      fileDownloadItem: this.fileDownloadItem
    });
  }

  get fileRetentionPeriodOptions(): DateOptions {
    return {
      manipulate: { amount: this.fileRetentionPeriod, unit: 'days' }
    };
  }
}
