import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output
} from '@angular/core';

import { FileDownloadItem } from '@ucap/domain-common';
import { FileInfo, FileEventJson } from '@ucap/domain-chat';

@Component({
  selector: 'ucap-chat-image-list-item-01',
  templateUrl: './image-list-item-01.component.html',
  styleUrls: ['./image-list-item-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ImageListItem01Component implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  @Input()
  fileInfo: FileInfo;

  @Input()
  expired = false;

  @Input()
  checked = false;

  @Input()
  checkable = true;

  @Input()
  fileDownloadItem: FileDownloadItem;

  @Output()
  changeCheck = new EventEmitter<{
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }>();

  @Output()
  save = new EventEmitter<{
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }>();

  @Output()
  openViewer = new EventEmitter<FileInfo>();

  @Output()
  delete = new EventEmitter<FileInfo>();

  showOverButtons = false;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onMouseover(event: MouseEvent): void {
    if (!this.expired) {
      this.showOverButtons = true;
    }
    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    this.showOverButtons = false;
    event.preventDefault();
    event.stopPropagation();
  }

  onChangeCheck(checked: boolean) {
    this.changeCheck.emit({
      checked,
      fileInfo: this.fileInfo,
      fileDownloadItem: this.fileDownloadItem
    });
  }

  onClickSave(event: MouseEvent) {
    event.stopPropagation();
    event.preventDefault();

    const param = {
      fileInfo: {
        // require parameters.
        fileName: this.fileInfo.name,
        attachmentSeq: this.fileInfo.seq
      } as FileEventJson,
      fileDownloadItem: this.fileDownloadItem,
      type: 'save'
    };
    this.save.emit(param);
  }
  onClickOpenViewer(event: MouseEvent) {
    event.stopPropagation();
    event.preventDefault();

    this.openViewer.emit(this.fileInfo);
  }
  onClickDelete(event: MouseEvent) {
    event.stopPropagation();
    event.preventDefault();

    this.delete.emit(this.fileInfo);
  }
}
