import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Directive,
  ContentChild,
  TemplateRef,
  ViewChild
} from '@angular/core';

import { FileInfo } from '@ucap/domain-chat';

import { PerfectScrollbarDirective } from 'ngx-perfect-scrollbar';

@Directive({
  selector: '[ucapChatImageListNode]'
})
export class ImageListNodeDirective {}

@Component({
  selector: 'ucap-chat-image-list',
  templateUrl: './image-list.component.html',
  styleUrls: ['./image-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ImageListComponent implements OnInit, OnDestroy {
  @Input()
  fileInfos: FileInfo[];

  @Input()
  vsItemSize: number;

  @ViewChild(PerfectScrollbarDirective, { static: false })
  psDirectiveRef: PerfectScrollbarDirective;

  @ContentChild(ImageListNodeDirective, {
    read: TemplateRef,
    static: false
  })
  nodeTemplate: TemplateRef<ImageListNodeDirective>;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }
}
