import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Directive,
  ContentChild,
  TemplateRef,
  ViewChild,
  ElementRef
} from '@angular/core';

import { ObjectUtil } from '@ucap/core';

import { UserInfo, UserInfoShort, RoomInfo } from '@ucap/domain-chat';

import { VirtualScrollViewportComponent } from '@ucap/ng-ui/scrolling';

export type ItemInfoTypes = UserInfo | UserInfoShort | RoomInfo;

@Directive({
  selector: '[ucapChatItemListNode]'
})
export class ChatItemListNodeDirective {}

@Component({
  selector: 'ucap-chat-item-list',
  templateUrl: './item-list.component.html',
  styleUrls: ['./item-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ChatItemListComponent implements OnInit, OnDestroy {
  @Input()
  itemInfos: ItemInfoTypes[];

  @Input()
  vsItemSize: number;

  @ContentChild(ChatItemListNodeDirective, {
    read: TemplateRef,
    static: false
  })
  nodeTemplate: TemplateRef<ChatItemListNodeDirective>;

  @ViewChild('container', { static: true })
  container: ElementRef<HTMLElement>;

  @ViewChild('cvsvList', { static: true })
  cvsvList: VirtualScrollViewportComponent;

  ObjectUtil = ObjectUtil;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }
}
