import {
  Component,
  OnInit,
  Output,
  Input,
  EventEmitter,
  ChangeDetectionStrategy
} from '@angular/core';

import { DeviceType, FileDownloadItem } from '@ucap/domain-common';
import { FileType, Info, FileEventJson, RoomInfo } from '@ucap/domain-chat';

import { StatusCode } from '@ucap/api';

import { SelectFileInfo } from '@ucap/ng-ui/viewer';

@Component({
  selector: 'ucap-chat-message-box-file',
  templateUrl: './file.component.html',
  styleUrls: ['./file.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class FileComponent implements OnInit {
  @Input()
  message: Info<FileEventJson>;

  @Input()
  roomInfo: RoomInfo;

  @Input()
  existFile = false;

  @Input()
  deviceType?: DeviceType;

  @Input()
  fileRetentionPeriod?: number;

  @Output()
  save = new EventEmitter<{
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }>();

  @Output()
  fileViewer = new EventEmitter<SelectFileInfo>();

  @Output()
  openFile = new EventEmitter<FileEventJson>();

  @Output()
  openFolder = new EventEmitter<FileEventJson>();

  fileInfo?: FileEventJson;
  fileDownloadItem: FileDownloadItem;
  errorMessage?: string;
  FileType = FileType;

  constructor() {}

  ngOnInit() {
    if (StatusCode.Success === this.message.sentMessageJson.statusCode) {
      this.fileInfo = this.message.sentMessageJson;
    } else {
      this.errorMessage =
        this.message.sentMessageJson.errorMessage || '[Error] System Error!!';
    }

    this.fileDownloadItem = new FileDownloadItem();
  }

  getExpiredFile() {
    if (
      !!this.roomInfo &&
      this.roomInfo.expiredFileStdSeq <= this.message.seq
    ) {
      return false;
    } else {
      return true;
    }
  }

  onClickFileViewer(fileInfo: FileEventJson) {
    if (!this.getExpiredFile()) {
      this.fileViewer.emit({ attachmentSeq: this.fileInfo.attachmentSeq });
    }
  }

  onSave(value: string) {
    if (!this.getExpiredFile()) {
      this.save.emit({
        fileInfo: this.fileInfo,
        fileDownloadItem: this.fileDownloadItem,
        type: value
      });
    }
  }
  onClickOpenFile(fileInfo: FileEventJson): void {
    if (!this.getExpiredFile()) {
      this.openFile.emit(fileInfo);
    }
  }
  onClickOpenFolder(fileInfo: FileEventJson): void {
    if (!this.getExpiredFile()) {
      this.openFolder.emit(fileInfo);
    }
  }
}
