import {
  Component,
  OnInit,
  Input,
  ChangeDetectionStrategy
} from '@angular/core';

import {
  EventType,
  Info,
  EventJson,
  RenameRoomEventJson,
  GuideForRoomTimerChangedEventJson,
  JoinEventJson
} from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

@Component({
  selector: 'ucap-chat-message-box-information',
  templateUrl: './information.component.html',
  styleUrls: ['./information.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class InformationComponent implements OnInit {
  @Input()
  message: Info<EventJson>;

  @Input()
  senderName?: string;

  EventType = EventType;

  constructor(private i18nService: I18nService) {}

  ngOnInit() {}

  protected getI18nForSir(names: string | string[]) {
    if (Array.isArray(names)) {
      return names.join(',');
    } else {
      return names;
    }
  }

  getOwnerForJoinEventJson() {
    return this.getI18nForSir(
      (this.message.sentMessageJson as JoinEventJson).owner
    );
  }
  getInviterForJoinEventJson() {
    return this.getI18nForSir(
      (this.message.sentMessageJson as JoinEventJson).inviter
    );
  }
  getCalcTimerForGuideForRoomTimerChanged() {
    const millisec =
      Number(
        (this.message.sentMessageJson as GuideForRoomTimerChangedEventJson).time
      ) * 1000;
    const second = this.i18nService.t('common:units.second');
    const minute = this.i18nService.t('common:units.minute');
    const hourFrom = this.i18nService.t('common:units.hourFrom');

    switch (millisec) {
      case 5000:
        return `5 ${second}`;
      case 10000:
        return `10 ${second}`;
      case 30000:
        return `30 ${second}`;
      case 60000:
        return `1 ${minute}`;
      case 300000:
        return `5 ${minute}`;
      case 600000:
        return `10 ${minute}`;
      case 1800000:
        return `30 ${minute}`;
      case 3600000:
        return `1 ${hourFrom}`;
      case 21600000:
        return `6 ${hourFrom}`;
      case 43200000:
        return `12 ${hourFrom}`;
      case 86400000:
        return `24 ${hourFrom}`;
      default:
        return `0 ${second}`;
    }
  }

  getRequesterForRenameRoom() {
    return (this.message.sentMessageJson as RenameRoomEventJson).requester;
  }
  getRoomNameForRenameRoom() {
    return (this.message.sentMessageJson as RenameRoomEventJson).roomName;
  }
}
