import {
  Component,
  OnInit,
  Input,
  Output,
  EventEmitter,
  ElementRef,
  AfterViewInit,
  ChangeDetectionStrategy
} from '@angular/core';

import { Info, MassTextEventJson } from '@ucap/domain-chat';

import { StatusCode } from '@ucap/api';

@Component({
  selector: 'ucap-chat-message-box-mass',
  templateUrl: './mass.component.html',
  styleUrls: ['./mass.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class MassComponent implements OnInit, AfterViewInit {
  @Input()
  message: Info<MassTextEventJson>;

  @Output()
  massDetail = new EventEmitter<number>();

  @Output()
  openLink = new EventEmitter<string>();

  content: string;
  eventMassSeq: number;
  detailButteonShow = true;

  constructor(private elementRef: ElementRef) {}

  ngOnInit() {
    try {
      if (StatusCode.Success === this.message.sentMessageJson.statusCode) {
        this.content = this.message.sentMessageJson.content;
      } else {
        this.content =
          this.message.sentMessageJson.errorMessage || '[Error] System Error!!';
        this.detailButteonShow = false;
      }

      if (!!this.message.sentMessageJson.massSeq) {
        this.eventMassSeq = this.message.sentMessageJson.massSeq;
      } else {
        this.detailButteonShow = false;
      }
    } catch (e) {
      this.detailButteonShow = false;
    }
  }

  ngAfterViewInit(): void {
    if (
      !!this.elementRef.nativeElement &&
      !!this.elementRef.nativeElement.querySelector('a')
    ) {
      const elements = this.elementRef.nativeElement.querySelectorAll('a');
      elements.forEach((element) => {
        element.addEventListener('click', this.onClickEvent.bind(this));
      });
    }
  }

  onClickEvent(event: MouseEvent) {
    this.openLink.emit((event.target as HTMLAnchorElement).text);
  }

  onClickDetailView() {
    this.massDetail.emit(this.eventMassSeq);
  }
}
