import {
  Component,
  OnInit,
  Input,
  EventEmitter,
  Output,
  ChangeDetectionStrategy
} from '@angular/core';

import { Info, PlanEventJson, PlanContentType } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

import moment from 'moment';

@Component({
  selector: 'ucap-chat-message-box-schedule',
  templateUrl: './schedule.component.html',
  styleUrls: ['./schedule.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ScheduleComponent implements OnInit {
  @Input()
  message: Info<PlanEventJson>;

  @Output()
  scheduleDetail = new EventEmitter<number>();

  PlanContentType = PlanContentType;
  date: any;
  endDate: any;

  constructor(private i18nService: I18nService) {}

  ngOnInit() {
    if (!!this.message && !!this.message.sentMessageJson) {
      if (!!this.message.sentMessageJson.date) {
        this.date = this._transeDateTime(this.message.sentMessageJson.date);
      }

      if (!!this.message.sentMessageJson.endDate) {
        this.endDate = this._transeDateTime(
          this.message.sentMessageJson.endDate
        );
      }
    }
  }

  getAlertLeftTime(): string {
    let content = this.message.sentMessageJson.contents;
    if (content.indexOf('PLAN_CONTENTS_MINUTE') > -1) {
      content = content.replace(
        'PLAN_CONTENTS_MINUTE',
        this.i18nService.t('common:units.minute')
      );
    } else if (content.indexOf('PLAN_CONTENTS_HOUR') > -1) {
      content = content.replace(
        'PLAN_CONTENTS_HOUR',
        this.i18nService.t('common:units.time')
      );
    }

    if (content.indexOf('PLAN_CONTENTS_AFTER') > -1) {
      content = content.replace(
        'PLAN_CONTENTS_AFTER',
        this.i18nService.t('chat:event.scheduleTypeSurfixLeft')
      );
    }

    content = this.i18nService.t('chat:event.scheduleTypePrefix') + content;

    return content;
  }

  onClickDetailView() {
    this.scheduleDetail.emit(this.message.sentMessageJson.planSeq);
  }

  private _transeDateTime(time: string): any {
    let tempDate = time
      .replace(/ /g, '')
      .replace(/\n/g, '')
      .replace(/(\([월,화,수,목,금,토,일]\))/g, '');
    if (tempDate.indexOf('오전') > -1) {
      tempDate = tempDate.replace('오전', ' ');
    } else if (tempDate.indexOf('오후') > -1) {
      tempDate = tempDate.replace('오후', ' ');
      const arr = tempDate.split(' ');
      const h = Number(arr[1].split(':')[0]) + 12;
      tempDate = arr[0] + ' ' + h + ':' + arr[1].split(':')[1];
    }

    let rtnDate: any = moment(tempDate).toDate();
    if (rtnDate === 'Invalid Date') {
      rtnDate = time.replace(/\n/g, '');
    }

    return rtnDate;
  }
}
