import {
  Component,
  OnInit,
  Input,
  Output,
  EventEmitter,
  ChangeDetectionStrategy
} from '@angular/core';

import { FileDownloadItem, DeviceType } from '@ucap/domain-common';
import { FileEventJson } from '@ucap/domain-chat';

import { DateOptions } from '@ucap/ng-ui/date';

@Component({
  selector: 'ucap-chat-message-box-video',
  templateUrl: './video.component.html',
  styleUrls: ['./video.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class VideoComponent implements OnInit {
  @Input()
  fileInfo: FileEventJson;

  @Input()
  fileDownloadItem: FileDownloadItem;

  @Input()
  existFile = false;

  @Input()
  deviceType: DeviceType;

  @Input()
  expired = false;

  @Input()
  fileRetentionPeriod?: number;

  @Output()
  save = new EventEmitter<string>();

  @Output()
  openFile = new EventEmitter();

  @Output()
  openFolder = new EventEmitter();

  @Output()
  openViewer = new EventEmitter();

  showOverButtons = false;

  DeviceType = DeviceType;

  constructor() {}

  ngOnInit() {}

  onClickSave() {
    if (!this.expired) {
      this.save.emit('save');
    }
  }
  onClickSaveAs() {
    if (!this.expired) {
      this.save.emit('saveAs');
    }
  }
  onClickOpenFile() {
    if (!this.expired) {
      this.openFile.emit();
    }
  }
  onClickOpenFolder() {
    if (!this.expired) {
      this.openFolder.emit();
    }
  }
  onClickOpenViewer() {
    if (!this.expired) {
      this.openViewer.emit();
    }
  }

  mouseEnter(event: MouseEvent): void {
    if (!this.expired) {
      this.showOverButtons = true;
    }
    event.stopPropagation();
    event.preventDefault();
  }
  mouseLeave(event: MouseEvent): void {
    this.showOverButtons = false;
    event.stopPropagation();
    event.preventDefault();
  }

  get fileRetentionPeriodOptions(): DateOptions {
    return {
      manipulate: { amount: this.fileRetentionPeriod, unit: 'days' }
    };
  }
}
