import {
  Component,
  OnInit,
  ChangeDetectionStrategy,
  Input,
  OnDestroy,
  EventEmitter,
  Output,
  ChangeDetectorRef
} from '@angular/core';

import { RoomInfo, RoomType, EventType } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

@Component({
  selector: 'ucap-chat-room-list-item-01',
  templateUrl: './room-list-item-01.component.html',
  styleUrls: ['./room-list-item-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class RoomListItem01Component implements OnInit, OnDestroy {
  @Input()
  roomInfo: RoomInfo;

  @Input()
  isCurrentRoom = false;

  @Input()
  profileImageRoot: string;

  @Input()
  defaultProfileImage: string;

  @Input()
  profileImage: string;

  @Input()
  roomName: string;

  @Input()
  checkable = false;

  @Input()
  checked = false;

  @Input()
  moreable = true;

  @Input()
  dateFormat?: string;

  @Output()
  toggleItem = new EventEmitter<{
    checked: boolean;
    roomInfo: RoomInfo;
  }>();

  @Output()
  openChatRoom = new EventEmitter<RoomInfo>();

  @Output()
  toggleAlarm = new EventEmitter<RoomInfo>();

  @Output()
  delRoom = new EventEmitter<RoomInfo>();

  RoomType = RoomType;

  constructor(
    private i18nService: I18nService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {}

  finalEventMessage(): string {
    if (!!this.roomInfo.isTimeRoom) {
      return this.i18nService.t('chat:event.isRoomTypeSecret');
    } else if (
      this.roomInfo.finalEventType === EventType.Character &&
      !!this.roomInfo.finalEventMessage &&
      this.roomInfo.finalEventMessage.trim().length === 0
    ) {
      return this.i18nService.t('chat:event.noRecentChat');
    } else {
      return this.roomInfo.finalEventMessage;
    }
  }

  onToggleItem(value: boolean): void {
    this.toggleItem.emit({
      checked: value,
      roomInfo: this.roomInfo
    });

    this.changeDetectorRef.detectChanges();
  }

  onOpenChatRoom(): void {
    this.openChatRoom.emit(this.roomInfo);
  }

  onToggleAlarm(): void {
    this.toggleAlarm.emit(this.roomInfo);
  }

  onDelRoom(): void {
    this.delRoom.emit(this.roomInfo);
  }
}
