import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter,
  ViewChild,
  ElementRef
} from '@angular/core';

import { FileUploadItem } from '@ucap/domain-common';
import { UserInfoSS } from '@ucap/domain-organization';
import { VersionInfo } from '@ucap/domain-authentication';
import { UserPermission } from '@ucap/domain-authorization';
import { OpenProfileOptions } from '@ucap/domain-group';
import { StatusBulkInfo } from '@ucap/domain-status';

@Component({
  selector: 'ucap-organization-profile-01',
  templateUrl: './profile-01.component.html',
  styleUrls: ['./profile-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class Profile01Component implements OnInit, OnDestroy {
  @Input()
  userPermission: UserPermission;

  @Input()
  versionInfo: VersionInfo;

  @Input()
  userInfo: UserInfoSS;

  @Input()
  nickName: string;

  @Input()
  profileImageRoot: string;

  @Input()
  defaultProfileImage: string;

  @Input()
  isMe: boolean;

  @Input()
  isBuddy: boolean;

  @Input()
  isFavorite: boolean;

  @Input()
  myMadn: string;

  @Input()
  canVideoConfernece: boolean;

  @Input()
  canBuddyToggleButton: boolean;

  @Input()
  openProfileOptions: OpenProfileOptions;

  @Input()
  presenceInfo: StatusBulkInfo;

  @Input()
  companyName: string;

  @Output()
  toggleFavorite: EventEmitter<{
    userInfo: UserInfoSS;
    isFavorite: boolean;
  }> = new EventEmitter();

  @Output()
  toggleBuddy: EventEmitter<{
    userInfo: UserInfoSS;
    isBuddy: boolean;
  }> = new EventEmitter();

  @Output()
  openChat: EventEmitter<UserInfoSS> = new EventEmitter();

  @Output()
  sendCall: EventEmitter<string> = new EventEmitter();

  @Output()
  sendSms: EventEmitter<string> = new EventEmitter();

  @Output()
  createConference: EventEmitter<number> = new EventEmitter();

  @Output()
  sendMessage: EventEmitter<UserInfoSS> = new EventEmitter();

  @Output()
  profileImageView: EventEmitter<UserInfoSS> = new EventEmitter();

  @Output()
  uploadProfileImage: EventEmitter<FileUploadItem> = new EventEmitter();

  @Output()
  changeIntro: EventEmitter<string> = new EventEmitter();

  @Output()
  changeNickname: EventEmitter<string> = new EventEmitter();

  @Output()
  settings: EventEmitter<void> = new EventEmitter();

  @ViewChild('profileImageFileInput', { static: false })
  profileImageFileInput: ElementRef<HTMLInputElement>;

  profileImageFileUploadItem: FileUploadItem;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangeFileInput() {
    this.profileImageFileUploadItem = FileUploadItem.fromFiles(
      this.profileImageFileInput.nativeElement.files
    )[0];
    this.uploadProfileImage.emit(this.profileImageFileUploadItem);

    this.profileImageFileInput.nativeElement.value = '';
  }

  onChangeIntro(intro: string) {
    this.changeIntro.emit(intro);
  }
  onChangeNickname(nickname: string) {
    this.changeNickname.emit(nickname);
  }

  onToggleFavorit() {
    this.isFavorite = !this.isFavorite;

    this.toggleFavorite.emit({
      userInfo: this.userInfo,
      isFavorite: this.isFavorite
    });
  }

  onClickAddBuddy() {
    this.toggleBuddy.emit({
      userInfo: this.userInfo,
      isBuddy: true
    });
  }

  onClickDelBuddy() {
    this.toggleBuddy.emit({
      userInfo: this.userInfo,
      isBuddy: false
    });
  }

  onOpenChat() {
    this.openChat.emit(this.userInfo);
  }
  onSendMessage() {
    this.sendMessage.emit(this.userInfo);
  }
  onSendCall(type: string) {
    let calleeNumber = '';

    if (type === 'LINE') {
      calleeNumber = this.userInfo.lineNumber;
    } else {
      calleeNumber = this.userInfo.hpNumber;
    }
    this.sendCall.emit(calleeNumber);
  }
  onSendSms() {
    this.sendSms.emit(this.userInfo.employeeNum);
  }
  onCreateConference() {
    this.createConference.emit(Number(this.userInfo.seq));
  }

  onProfileImageView(userInfo: UserInfoSS) {
    this.profileImageView.emit(userInfo);
  }

  onSettings(event: MouseEvent) {
    this.settings.emit();
  }

  getShowBuddyToggleBtn(type: 'DEL' | 'ADD'): boolean {
    let rtn = false;
    if (!this.canBuddyToggleButton) {
      return false;
    }

    if (type === 'ADD') {
      if (!this.isBuddy) {
        rtn = true;
      }
    } else if (type === 'DEL') {
      if (!!this.isBuddy) {
        rtn = true;
      }
    }
    return rtn;
  }

  getUserIntro(info: UserInfoSS): string {
    if (!!info && !!info.intro) {
      if (info.intro.trim().localeCompare('') === 0) {
        return info.intro.trim();
      } else {
        return info.intro;
      }
    }

    return '';
  }
  getDisabledBtn(type: string): boolean {
    if (!this.myMadn || this.myMadn.trim().length === 0) {
      if (type === 'LINE' || type === 'MOBILE') {
        return true;
      }
    }

    if (type === 'LINE') {
      if (
        !!this.userInfo &&
        !!this.userInfo.lineNumber &&
        this.userInfo.lineNumber.trim().length > 0
      ) {
        return false;
      } else {
        return true;
      }
    } else if (type === 'MOBILE') {
      if (
        !!this.userInfo &&
        !!this.userInfo.hpNumber &&
        this.userInfo.hpNumber.trim().length > 0
      ) {
        return false;
      } else {
        return true;
      }
    }
    // else if (type === 'SMS') {
    //   const smsUtils = new smsUtils(
    //     this.sessionStorageService,
    //     this.nativeService
    //   );
    //   return !smsUtils.getAuthSms();
    // }

    return true;
  }

  getUserNameForEn(): string {
    if (
      !!this.userInfo &&
      !!this.userInfo.nameEn &&
      this.userInfo.nameEn.trim() !== ''
    ) {
      return '(' + this.userInfo.nameEn + ')';
    }

    return '';
  }
}
