import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { UserInfoSS } from '@ucap/domain-organization';
import { PresenceType, StatusBulkInfo } from '@ucap/domain-status';

import { I18nService } from '@ucap/ng-i18n';

import { PresenceUtil } from '../utils/presence.util';

@Component({
  selector: 'ucap-organization-profile-image-01',
  templateUrl: './profile-image-01.component.html',
  styleUrls: ['./profile-image-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileImage01Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoSS;

  @Input()
  presenceInfo: StatusBulkInfo;

  @Input()
  profileImageRoot: string;

  @Input()
  defaultProfileImage: string;

  @Output()
  openProfile: EventEmitter<UserInfoSS> = new EventEmitter();

  PresenceType = PresenceType;
  PresenceUtil = PresenceUtil;

  private ngOnDestroySubject: Subject<void>;

  constructor(
    private i18nService: I18nService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickOpenProfile(event: MouseEvent): void {
    event.preventDefault();
    event.stopPropagation();

    this.openProfile.emit(this.userInfo);
  }

  getProfileImageFile(userInfo: UserInfoSS): string | undefined {
    if (!!userInfo) {
      return userInfo.profileImageFile;
    } else {
      return undefined;
    }
  }

  getStatusMessage(presenceType: PresenceType): string {
    if (PresenceUtil.isOnline(this.presenceInfo, presenceType)) {
      return this.i18nService.t('organization:presence.online');
    } else if (PresenceUtil.isOffline(this.presenceInfo, presenceType)) {
      return this.i18nService.t('organization:presence.offline');
    } else if (
      PresenceUtil.isAway(this.presenceInfo, presenceType) ||
      PresenceUtil.isAway2(this.presenceInfo, presenceType)
    ) {
      return this.i18nService.t('organization:presence.away');
    } else if (PresenceUtil.isBusy(this.presenceInfo, presenceType)) {
      return this.presenceInfo.statusMessage;
    } else {
      return '';
    }
  }
}
