import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { UserInfoSS } from '@ucap/domain-organization';

import { CallType } from '../types/call.type';

@Component({
  selector: 'ucap-organization-profile-list-item-01',
  templateUrl: './profile-list-item-01.component.html',
  styleUrls: ['./profile-list-item-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListItem01Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoSS;

  @Input()
  authCall = false;

  @Input()
  checkable = true;

  @Input()
  checked = false;

  @Output()
  changeCheck = new EventEmitter<{
    checked: boolean;
    userInfo: UserInfoSS;
  }>();

  @Output()
  sendCall = new EventEmitter<{
    type: CallType;
  }>();

  CallType = CallType;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangeCheck(checked: boolean): void {
    this.changeCheck.emit({
      checked,
      userInfo: this.userInfo
    });
  }

  onSendCall(event: MouseEvent, type: CallType): void {
    event.preventDefault();
    event.stopPropagation();

    this.sendCall.emit({
      type
    });
  }
}
