import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { UserInfoF } from '@ucap/domain-organization';

@Component({
  selector: 'ucap-organization-profile-list-item-02',
  templateUrl: './profile-list-item-02.component.html',
  styleUrls: ['./profile-list-item-02.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListItem02Component implements OnInit, OnDestroy {
  @Input()
  userInfo: UserInfoF;

  @Input()
  authCall = false;

  @Input()
  checkable = true;

  @Input()
  checked = false;

  @Output()
  changeCheck = new EventEmitter<{
    checked: boolean;
    userInfo: UserInfoF;
  }>();

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangeCheck(checked: boolean): void {
    this.changeCheck.emit({
      checked,
      userInfo: this.userInfo
    });
  }
}
