import { Subject } from 'rxjs';
import { takeUntil, take, map, withLatestFrom, tap } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  HostBinding,
  Output,
  EventEmitter,
  Directive,
  ContentChild,
  TemplateRef
} from '@angular/core';

import { MatRadioChange } from '@angular/material/radio';

import { User } from '@ucap/domain-organization';
import { VersionInfo } from '@ucap/domain-authentication';

import { LogService } from '@ucap/ng-logger';

@Directive({
  selector: '[ucapOrganizationProfileMenu01Profile]'
})
export class ProfileMenu01ProfileDirective {}

@Directive({
  selector: '[ucapOrganizationProfileMenu01ProfileBelow]'
})
export class ProfileMenu01ProfileBelowDirective {}

@Directive({
  selector: '[ucapOrganizationProfileMenu01IntroBelow]'
})
export class ProfileMenu01IntroBelowDirective {}

@Directive({
  selector: '[ucapOrganizationProfileMenu01StatusMessageBelow]'
})
export class ProfileMenu01StatusMessageBelowDirective {}

@Component({
  selector: 'ucap-organization-profile-menu-01',
  templateUrl: './profile-menu-01.component.html',
  styleUrls: ['./profile-menu-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileMenu01Component implements OnInit, OnDestroy {
  @HostBinding('class')
  cssClass = 'ucap-organization-profile-menu-01';

  @Input()
  set user(user: User) {
    this._user = user;
    this._userIntro =
      !user || !user.info || !user.info.intro || '' === user.info.intro.trim()
        ? ''
        : user.info.intro.trim();
  }
  get user() {
    return this._user;
  }
  // tslint:disable-next-line: variable-name
  _user: User;

  @Input()
  versionInfo: VersionInfo;

  @Input()
  idleCheckTime: number;

  @Input()
  placeholderForUserIntro: string;

  @Input()
  idleTimePeriods: number[];

  @Output()
  changedIntro: EventEmitter<string> = new EventEmitter();

  @Output()
  changedStatusMessage: EventEmitter<{
    index: number;
    message: string;
  }> = new EventEmitter();

  @Output()
  selectedStatus: EventEmitter<
    'online' | 'away' | 'busy1' | 'busy2' | 'busy3'
  > = new EventEmitter();

  @Output()
  changedIdleCheckTime: EventEmitter<number> = new EventEmitter();

  @Output()
  logout: EventEmitter<void> = new EventEmitter();

  @Output()
  exit: EventEmitter<void> = new EventEmitter();

  @ContentChild(ProfileMenu01ProfileDirective, {
    read: TemplateRef,
    static: false
  })
  profileTemplate: TemplateRef<ProfileMenu01ProfileDirective>;

  @ContentChild(ProfileMenu01ProfileBelowDirective, {
    read: TemplateRef,
    static: false
  })
  profileBelowTemplate: TemplateRef<ProfileMenu01ProfileBelowDirective>;

  @ContentChild(ProfileMenu01IntroBelowDirective, {
    read: TemplateRef,
    static: false
  })
  introBelowTemplate: TemplateRef<ProfileMenu01IntroBelowDirective>;

  @ContentChild(ProfileMenu01StatusMessageBelowDirective, {
    read: TemplateRef,
    static: false
  })
  statusMessageBelowTemplate: TemplateRef<
    ProfileMenu01StatusMessageBelowDirective
  >;

  // tslint:disable-next-line: variable-name
  _userIntro: string;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onApplyIntro(intro: string) {
    this.changedIntro.emit(intro);
  }

  onCancelIntro(event: Event) {}

  onApplyStatusMessage(index: number, message: string) {
    this.changedStatusMessage.emit({ index, message });
  }

  onCancelStatusMessage(event: Event, index: number) {}

  onClickStatus(status: 'online' | 'away' | 'busy1' | 'busy2' | 'busy3') {
    this.selectedStatus.emit(status);
  }

  onChangeIdleCheckTime(event: MatRadioChange) {
    this.changedIdleCheckTime.emit(Number(event.value));
  }

  onClickLogout(event: Event) {
    this.logout.emit();
  }

  onClickExit(event: Event) {
    this.exit.emit();
  }
}
