import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Output,
  EventEmitter,
  Input,
  ElementRef,
  Self,
  ViewChildren,
  QueryList
} from '@angular/core';

import { MatTooltip } from '@angular/material/tooltip';

import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { UserPermission } from '@ucap/domain-authorization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { ProfileMenuType } from '../types/profile-menu.type';

export type UserInfoTypes = UserInfo | UserInfoSS | UserInfoF | UserInfoDN;

@Component({
  selector: 'ucap-organization-profile-menu-02',
  templateUrl: './profile-menu-02.component.html',
  styleUrls: ['./profile-menu-02.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileMenu02Component implements OnInit, OnDestroy {
  /**
   * 1: normal, 2: chat>roomuserlist, 3:call>organization>userlist
   */
  @Input()
  menuButtonType: string;

  @Input()
  user: User;

  @Input()
  userPermission: UserPermission;

  @Input()
  userInfo: UserInfoTypes;

  @Input()
  groupInfo: GroupInfoDetail;

  @Input()
  isShowMoreMenu: boolean;

  @Input()
  isMe: boolean;

  @Input()
  isShowForce: boolean;

  @Output()
  clickProfileMenu = new EventEmitter<{
    menuType: ProfileMenuType;
    userInfo: UserInfoTypes;
  }>();

  @Output()
  clickMoreMenu = new EventEmitter<{
    event: MouseEvent;
    userInfo: UserInfoTypes;
    group: GroupInfoDetail;
    rect: any;
  }>();

  @ViewChildren(MatTooltip)
  tooltipQueryList: QueryList<MatTooltip>;

  private ngOnDestroySubject: Subject<void> = new Subject();

  ProfileMenuType = ProfileMenuType;

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    @Self() private elementRef: ElementRef
  ) {}

  ngOnInit(): void {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    this._closeTooltip();
  }

  onClickProfileMenu(event: MouseEvent, menuType: ProfileMenuType) {
    event.stopPropagation();
    this._closeTooltip();

    this.clickProfileMenu.emit({ menuType, userInfo: this.userInfo });
  }

  onClickMoreMenu(event: MouseEvent) {
    event.stopPropagation();
    this._closeTooltip();

    const rect = this.elementRef.nativeElement.getBoundingClientRect();

    this.clickMoreMenu.emit({
      event,
      userInfo: this.userInfo,
      group: this.groupInfo,
      rect
    });
  }

  getDisabledBtn(type: ProfileMenuType): boolean {
    if (!this.user || !this.user.madn || this.user.madn.trim().length === 0) {
      if (type === ProfileMenuType.office || type === ProfileMenuType.mobile) {
        return true;
      }
    }

    if (type === ProfileMenuType.office) {
      if (
        !!this.userInfo &&
        !!this.userInfo.lineNumber &&
        this.userInfo.lineNumber.trim().length > 0
      ) {
        return false;
      } else {
        return true;
      }
    } else if (type === ProfileMenuType.mobile) {
      if (
        !!this.userInfo &&
        !!this.userInfo.hpNumber &&
        this.userInfo.hpNumber.trim().length > 0
      ) {
        return false;
      } else {
        return true;
      }
    } else if (type === ProfileMenuType.videoConference) {
      if (!!this.userPermission && !!this.userPermission.canVideoConference) {
        return false;
      } else {
        return true;
      }
    }
    // else if (type === 'SMS') {
    //   const smsUtils = new smsUtils(
    //     this.sessionStorageService,
    //     this.nativeService
    //   );
    //   return !smsUtils.getAuthSms();
    // }

    return true;
  }

  private _closeTooltip() {
    if (!!this.tooltipQueryList && this.tooltipQueryList.length > 0) {
      this.tooltipQueryList.forEach((component) => {
        component.hide();
      });
    }
  }
}
