import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Output,
  EventEmitter,
  Input,
  Directive,
  ContentChild,
  TemplateRef
} from '@angular/core';

import { MatChipEvent } from '@angular/material/chips';

import { UserInfo } from '@ucap/domain-organization';

export interface ProfileSelection {
  userInfo: UserInfo;
  color?: string;
  selectable?: boolean;
  selected?: boolean;
  removable?: boolean;
}

@Directive({
  selector: '[ucapOrganizationProfileSelection01Header]'
})
export class ProfileSelection01HeaderDirective {}

@Directive({
  selector: '[ucapOrganizationProfileSelection01Actions]'
})
export class ProfileSelection01ActionsDirective {}

@Component({
  selector: 'ucap-organization-profile-selection-01',
  templateUrl: './profile-selection-01.component.html',
  styleUrls: ['./profile-selection-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileSelection01Component implements OnInit, OnDestroy {
  @Input()
  set userInfoList(value: UserInfo[]) {
    this._userInfoList = value;
  }
  // tslint:disable-next-line: variable-name
  _userInfoList: UserInfo[];

  @Input()
  removableFor: (userInfo: UserInfo) => boolean;

  @Input()
  colorFor: (userInfo: UserInfo) => string;

  @Output()
  removed: EventEmitter<UserInfo> = new EventEmitter();

  @ContentChild(ProfileSelection01HeaderDirective, {
    read: TemplateRef,
    static: false
  })
  headerTemplate: TemplateRef<ProfileSelection01HeaderDirective>;

  @ContentChild(ProfileSelection01ActionsDirective, {
    read: TemplateRef,
    static: false
  })
  actionsTemplate: TemplateRef<ProfileSelection01ActionsDirective>;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  _colorFor(userInfo: UserInfo): string {
    if (!this.colorFor) {
      return undefined;
    }
    return this.colorFor(userInfo);
  }

  _removableFor(userInfo: UserInfo): boolean {
    if (!this.removableFor) {
      return undefined;
    }
    return this.removableFor(userInfo);
  }

  _onRemoved(event: MatChipEvent, userInfo: UserInfo) {
    this.removed.emit(userInfo);
  }
}
