import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ViewChild,
  ElementRef
} from '@angular/core';

import { MatSelectChange, MatSelect } from '@angular/material/select';

import { Company } from '@ucap/domain-organization';

@Component({
  selector: 'ucap-organization-search-for-tenant',
  templateUrl: './search-for-tenant.component.html',
  styleUrls: ['./search-for-tenant.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SearchForTenantComponent implements OnInit, OnDestroy {
  @Input()
  companyList: Company[];

  @Input()
  defaultCompany: string;

  @Input()
  defaultSearchWord: string;

  @Input()
  placeholder: string;

  @Input()
  multiple: boolean;

  @Output()
  changed: EventEmitter<{
    companyCode: string;
    searchWord: string;
  }> = new EventEmitter();

  @Output()
  canceled: EventEmitter<void> = new EventEmitter();

  @Output()
  keyDownBackspace: EventEmitter<Event> = new EventEmitter();

  @ViewChild('selectSearchCompany', { static: true })
  selectSearchCompany: MatSelect;

  @ViewChild('inputSearchWord', { static: true })
  inputSearchWord: ElementRef<HTMLInputElement>;

  showRemoveBtn = false;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.complete();
    }
  }

  onSelectionChangeTenant(event: MatSelectChange) {
    // this.changedSearchData();
  }
  onKeyupBackspace(event: Event) {
    if (this.inputSearchWord.nativeElement.value === '') {
      this.keyDownBackspace.emit(event);
    }
    event.stopPropagation();
  }

  onKeyupEnter(event: Event) {
    this.changedSearchData();
  }

  onClickSearch(event: Event) {
    this.changedSearchData();
  }

  onClickCancel(event: Event) {
    this.changeSearchword('');
    this.canceled.emit();
    event.stopPropagation();
  }

  changeSearchword(search: string) {
    if (search.localeCompare('') === 0) {
      this.inputSearchWord.nativeElement.value = '';
      this.showRemoveBtn = false;
    } else {
      this.showRemoveBtn = true;
    }
  }

  private changedSearchData() {
    this.changed.emit({
      companyCode: this.selectSearchCompany.value,
      searchWord: this.inputSearchWord.nativeElement.value
    });
  }
}
