import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatButtonModule } from '@angular/material/button';
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatChipsModule } from '@angular/material/chips';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatRadioModule } from '@angular/material/radio';
import { MatRippleModule } from '@angular/material/core';
import { MatSelectModule } from '@angular/material/select';
import { MatTreeModule } from '@angular/material/tree';
import { MatTooltipModule } from '@angular/material/tooltip';

import { PerfectScrollbarModule } from 'ngx-perfect-scrollbar';

import { UCAP_I18N_NAMESPACE, I18nModule } from '@ucap/ng-i18n';

import { UiCoreModule } from '@ucap/ng-ui/core';
import { UiInputModule } from '@ucap/ng-ui/input';
import { UiPhoneModule } from '@ucap/ng-ui/phone';
import { UiscrollingModule } from '@ucap/ng-ui/scrolling';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import { Profile01Component } from './components/profile-01.component';
import { ProfileListItem01Component } from './components/profile-list-item-01.component';
import { ProfileListItem02Component } from './components/profile-list-item-02.component';
import { ProfileImage01Component } from './components/profile-image-01.component';
import {
  ProfileListComponent,
  ProfileListNodeDirective
} from './components/profile-list.component';

import { ProfileMenu02Component } from './components/profile-menu-02.component';

import {
  ProfileMenu01Component,
  ProfileMenu01IntroBelowDirective,
  ProfileMenu01ProfileBelowDirective,
  ProfileMenu01ProfileDirective,
  ProfileMenu01StatusMessageBelowDirective
} from './components/profile-menu-01.component';

import {
  ProfileSelection01Component,
  ProfileSelection01HeaderDirective,
  ProfileSelection01ActionsDirective
} from './components/profile-selection-01.component';
import { SearchForTenantComponent } from './components/search-for-tenant.component';
import { TreeComponent } from './components/tree.component';

import { TranslatePipe } from './pipes/translate.pipe';

import { TranslateService } from './services/translate.service';

const COMPONENTS = [
  Profile01Component,
  ProfileListItem01Component,
  ProfileListItem02Component,
  ProfileListComponent,
  ProfileImage01Component,
  ProfileMenu01Component,
  ProfileMenu02Component,
  ProfileSelection01Component,
  SearchForTenantComponent,
  TreeComponent
];
const DIALOGS = [];
const PIPES = [TranslatePipe];
const DIRECTIVES = [
  ProfileListNodeDirective,

  ProfileMenu01IntroBelowDirective,
  ProfileMenu01ProfileBelowDirective,
  ProfileMenu01ProfileDirective,
  ProfileMenu01StatusMessageBelowDirective,

  ProfileSelection01HeaderDirective,
  ProfileSelection01ActionsDirective
];
const SERVICES = [TranslateService];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class OrganizationUiRootModule {}

@NgModule({
  imports: [
    CommonModule,
    FlexLayoutModule,

    MatButtonModule,
    MatCheckboxModule,
    MatChipsModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatRadioModule,
    MatRippleModule,
    MatSelectModule,
    MatTreeModule,
    MatTooltipModule,

    PerfectScrollbarModule,

    I18nModule,

    UiCoreModule,
    UiInputModule,
    UiPhoneModule,
    UiscrollingModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS],
  providers: [
    {
      provide: UCAP_I18N_NAMESPACE,
      useValue: ['organization']
    }
  ]
})
export class OrganizationUiModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<OrganizationUiRootModule> {
    return {
      ngModule: OrganizationUiRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
