import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MatIconModule } from '@angular/material/icon';

import { I18nModule } from '@ucap/ng-i18n';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import { AlertDialogComponent } from './dialogs/alert.dialog.component';
import { ConfirmDialogComponent } from './dialogs/confirm.dialog.component';

import { ClickDebounceDirective } from './directives/click-debounce.directive';
import { ClickOutsideDirective } from './directives/click-outside.directive';
import { ImageDirective } from './directives/image.directive';
import { VarDirective } from './directives/var.directive';

import { BytesPipe } from './pipes/bytes.pipe';
import { EmptyPipe } from './pipes/empty.pipe';
import { HtmlToLinefeedPipe, LinefeedToHtmlPipe } from './pipes/linefeed.pipe';
import { LinkyPipe } from './pipes/linky.pipe';
import { SafeHtmlPipe } from './pipes/safe-html.pipe';

import { ClipboardService } from './services/clipboard.service';

const COMPONENTS = [];
const DIALOGS = [AlertDialogComponent, ConfirmDialogComponent];
const PIPES = [
  BytesPipe,
  EmptyPipe,
  HtmlToLinefeedPipe,
  LinefeedToHtmlPipe,
  LinkyPipe,
  SafeHtmlPipe
];
const DIRECTIVES = [
  ClickDebounceDirective,
  ClickOutsideDirective,
  ImageDirective,
  VarDirective
];
const SERVICES = [ClipboardService];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class UiCoreRootModule {}

@NgModule({
  imports: [
    CommonModule,
    FlexLayoutModule,
    MatButtonModule,
    MatCardModule,
    MatIconModule,
    I18nModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIALOGS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS]
})
export class UiCoreModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<UiCoreRootModule> {
    return {
      ngModule: UiCoreRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
