import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatDatepickerModule } from '@angular/material/datepicker';

import { I18nModule } from '@ucap/ng-i18n';
import { UiInputModule } from '@ucap/ng-ui/input';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import { PickDateComponent } from './components/pick-date.component';
import { PickTimeComponent } from './components/pick-time.component';

import { DatePipe } from './pipes/date.pipe';
import { SecondsToMinutesPipe } from './pipes/seconds-to-minutes.pipe';

import { DateService } from './services/date.service';

const COMPONENTS = [PickDateComponent, PickTimeComponent];
const DIRECTIVES = [];
const PIPES = [DatePipe, SecondsToMinutesPipe];
const DIALOGS = [];
const SERVICES = [DateService];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class UiDateRootModule {}

@NgModule({
  imports: [
    CommonModule,
    FlexLayoutModule,

    MatDatepickerModule,

    I18nModule,
    UiInputModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIALOGS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS]
})
export class UiDateModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<UiDateRootModule> {
    return {
      ngModule: UiDateRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
