import { Observable } from 'rxjs';

import { InjectionToken } from '@angular/core';

import { VirtualScrollViewportComponent } from '../components/virtual-scroll-viewport';

export type VirtualScrollToAlign =
  | 'auto'
  | 'smart'
  | 'center'
  | 'start'
  | 'end';
export type VirtualScrollKeyType = string | number;
export type VirtualScrollKeyOfFunction<T> = (item: T) => VirtualScrollKeyType;
export interface VirtualScrollMeasureSize {
  key: VirtualScrollKeyType;
  size: number;
}

/** The injection token used to specify the virtual scrolling strategy. */
export const VIRTUAL_SCROLL_STRATEGY = new InjectionToken<
  VirtualScrollStrategy
>('UCAP_VIRTUAL_SCROLL_STRATEGY');

/** A strategy that dictates which items should be rendered in the viewport. */
export interface VirtualScrollStrategy {
  /** Emits when the index of the first element visible in the viewport changes. */
  scrolledIndexChange: Observable<number>;

  /**
   * Attaches this scroll strategy to a viewport.
   * @param viewport The viewport to attach this strategy to.
   */
  attach(viewport: VirtualScrollViewportComponent): void;

  /** Detaches this scroll strategy from the currently attached viewport. */
  detach(): void;

  useMeasureSize(): boolean;

  measureSize(sizes: VirtualScrollMeasureSize[]): void;

  measuredSizeOf(key: VirtualScrollKeyType): number;

  /** Called when the viewport is scrolled (debounced using requestAnimationFrame). */
  onContentScrolled(): void;

  /** Called when the length of the data changes. */
  onDataChanged(): void;

  /** Called when the range of items rendered in the DOM has changed. */
  onContentRendered(): void;

  /** Called when the offset of the rendered items changed. */
  onRenderedOffsetChanged(): void;

  /**
   * Scroll to the offset for the given index.
   * @param index The index of the element to scroll to.
   * @param behavior The ScrollBehavior to use when scrolling.
   */
  scrollToIndex(
    index: number,
    scrollOffset: number,
    align: VirtualScrollToAlign,
    behavior: ScrollBehavior
  ): void;

  offsetForIndex(
    index: number,
    scrollOffset: number,
    align: VirtualScrollToAlign
  ): number;
}
