import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatRippleModule } from '@angular/material/core';

import { I18nModule } from '@ucap/ng-i18n';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import { VirtualScrollViewportComponent } from './components/virtual-scroll-viewport';

import {
  VirtualScrollHeaderDirective,
  VirtualScrollFooterDirective
} from './components/virtual-scroll-viewport';
import { ScrollableDirective } from './directives/scrollable';
import { VirtualForOfDirective } from './directives/virtual-for-of';
import { FixedSizeVirtualScrollDirective } from './strategies/fixed-size-virtual-scroll';
import { AutoSizeVirtualScrollDirective } from './strategies/auto-size-virtual-scroll';
import { MeasureSizeVirtualScrollDirective } from './strategies/measure-size-virtual-scroll';

const COMPONENTS = [VirtualScrollViewportComponent];
const DIRECTIVES = [
  ScrollableDirective,
  VirtualForOfDirective,
  VirtualScrollHeaderDirective,
  VirtualScrollFooterDirective,
  FixedSizeVirtualScrollDirective,
  AutoSizeVirtualScrollDirective,
  MeasureSizeVirtualScrollDirective
];
const PIPES = [];
const DIALOGS = [];
const SERVICES = [];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class UiscrollingRootModule {}

@NgModule({
  imports: [CommonModule, FlexLayoutModule, MatRippleModule, I18nModule],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIALOGS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS]
})
export class UiscrollingModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<UiscrollingRootModule> {
    return {
      ngModule: UiscrollingRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
