import { Component, OnInit, Input, Output, EventEmitter } from '@angular/core';

import { FileDownloadItem } from '@ucap/domain-common';
import { FileInfo, isSound, FileType } from '@ucap/domain-chat';

import { FileViewerType } from '../types/file-viewer.type';
import { SelectFileInfo } from '../models/select-file-info';

@Component({
  selector: 'ucap-media-viewer',
  templateUrl: './media-viewer.component.html',
  styleUrls: ['./media-viewer.component.scss']
})
export class MediaViewerComponent implements OnInit {
  @Input()
  set fileInfo(v: { fileInfos: FileInfo[]; selectFileInfo: SelectFileInfo }) {
    this.fileInfos = v.fileInfos.sort((a, b) => a.eventSeq - b.eventSeq);

    this.fileInfoIndex = this.fileInfos.findIndex(
      (f) => f.sentMessageJson.attachmentSeq === v.selectFileInfo.attachmentSeq
    );
    this.bundleIndex = v.selectFileInfo.index;

    this.setCuttentFileInfo();
  }

  @Input()
  fileDownloadUrl: (attachmentSeq: number) => string;

  @Output()
  download = new EventEmitter<{
    attachmentSeq?: number;
    downloadUrl?: string;
    fileName: string;
    fileDownloadItem: FileDownloadItem;
    roomSeq?: number;
    eventSeq?: number;
  }>();

  @Output()
  closed = new EventEmitter<void>();

  fileInfos: FileInfo[];
  currentFileInfo: FileInfo;
  fileInfoIndex: number;
  bundleIndex: number;
  FileType = FileType;
  FileViewerType = FileViewerType;

  get downloadUrl() {
    return this.fileDownloadUrl(this.currentFileInfo.seq);
  }

  get fileName() {
    return this.currentFileInfo.sentMessageJson.fileName;
  }

  constructor() {}

  ngOnInit() {}

  detectFileViewerType(fileInfo: FileInfo): FileViewerType {
    switch (fileInfo.type) {
      case FileType.Image:
        return FileViewerType.Image;
      case FileType.Sound:
        return FileViewerType.Sound;
      case FileType.Video:
        return FileViewerType.Video;
      default:
        if (isSound(fileInfo)) {
          return FileViewerType.Sound;
        }
        return FileViewerType.Binary;
    }
  }

  onDownload(fileDownloadItem: FileDownloadItem): void {
    const fileJsonInfo = this.currentFileInfo.sentMessageJson;
    this.download.emit({
      attachmentSeq: this.currentFileInfo.sentMessageJson.attachmentSeq,
      fileName: this.currentFileInfo.sentMessageJson.fileName,
      fileDownloadItem
    });
  }

  onClosedViewer(): void {
    this.closed.emit();
  }

  onClickPrev() {
    if (0 === this.fileInfoIndex) {
      return;
    }

    this.fileInfoIndex--;

    this.setCuttentFileInfo(false);
  }

  onClickNext() {
    if (this.fileInfos.length - 1 === this.fileInfoIndex) {
      return;
    }
    this.fileInfoIndex++;
    this.setCuttentFileInfo(true);
  }

  private setCuttentFileInfo(direction?: boolean) {
    this.currentFileInfo = this.fileInfos[this.fileInfoIndex];
    // if (
    //   FileType.Bundle === this.currentFileInfo.type &&
    //   undefined === this.bundleIndex
    // ) {
    //   if (undefined === direction) {
    //     return;
    //   }
    //   const fileJsonInfo = this.currentFileInfo
    //     .sentMessageJson as BundleImageEventJson;
    //   if (direction) {
    //     // Next
    //     this.bundleIndex = 0;
    //   } else {
    //     // Prev
    //     this.bundleIndex = fileJsonInfo.fileCount - 1;
    //   }
    // }
  }
}
