import { NgModule, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';

import { FlexLayoutModule } from '@angular/flex-layout';

import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { MatSliderModule } from '@angular/material/slider';
import { MatToolbarModule } from '@angular/material/toolbar';
import { MatTooltipModule } from '@angular/material/tooltip';

import { I18nModule } from '@ucap/ng-i18n';
import { UiDateModule } from '@ucap/ng-ui/date';

import { ModuleConfig } from './config/module-config';
import { _MODULE_CONFIG } from './config/token';

import { BinaryViewerComponent } from './components/file-viewer/binary-viewer.component';
import { DocumentViewerComponent } from './components/file-viewer/document-viewer.component';
import { ImageViewerComponent } from './components/file-viewer/image-viewer.component';
import { SoundViewerComponent } from './components/file-viewer/sound-viewer.component';
import { VideoViewerComponent } from './components/file-viewer/video-viewer.component';
import { FileViewerComponent } from './components/file-viewer.component';
import { MediaViewerComponent } from './components/media-viewer.component';

const COMPONENTS = [
  BinaryViewerComponent,
  DocumentViewerComponent,
  ImageViewerComponent,
  SoundViewerComponent,
  VideoViewerComponent,
  FileViewerComponent,
  MediaViewerComponent
];
const DIRECTIVES = [];
const PIPES = [];
const DIALOGS = [];
const SERVICES = [];

@NgModule({
  declarations: [],
  imports: [],
  exports: []
})
export class UiViewerRootModule {}

@NgModule({
  imports: [
    CommonModule,
    FlexLayoutModule,

    MatButtonModule,
    MatIconModule,
    MatProgressBarModule,
    MatSliderModule,
    MatToolbarModule,
    MatTooltipModule,

    I18nModule,
    UiDateModule
  ],
  exports: [...COMPONENTS, ...DIRECTIVES, ...PIPES],
  declarations: [...COMPONENTS, ...DIALOGS, ...DIRECTIVES, ...PIPES],
  entryComponents: [...DIALOGS]
})
export class UiViewerModule {
  public static forRoot(
    config: ModuleConfig
  ): ModuleWithProviders<UiViewerRootModule> {
    return {
      ngModule: UiViewerRootModule,
      providers: [{ provide: _MODULE_CONFIG, useValue: config }, ...SERVICES]
    };
  }
}
